from ecl.eclfile import EclKW, openFortIO, FortIO
from ecl import EclDataType
from ecl.grid import EclGrid
import os
import segyio
from segyio import TraceField
import numpy as np


def create_init(grid, case):
    poro = EclKW("PORO", grid.getNumActive(), EclDataType.ECL_FLOAT)
    porv = EclKW("PORV", grid.getGlobalSize(), EclDataType.ECL_FLOAT)

    with openFortIO("%s.INIT" % case, mode=FortIO.WRITE_MODE) as f:
        poro.fwrite(f)
        porv.fwrite(f)


def create_restart(grid, case, rporv=None):
    with openFortIO("%s.UNRST" % case, mode=FortIO.WRITE_MODE) as f:
        seq_hdr = EclKW("SEQNUM", 1, EclDataType.ECL_FLOAT)
        seq_hdr[0] = 1
        p = EclKW("PRESSURE", grid.getNumActive(), EclDataType.ECL_FLOAT)
        p.assign(1)

        for i, _ in enumerate(p):
            p[i] = 10

        header = EclKW("INTEHEAD", 67, EclDataType.ECL_INT)
        header[64] = 1
        header[65] = 1
        header[66] = 2000

        seq_hdr.fwrite(f)
        header.fwrite(f)
        p.fwrite(f)

        if rporv:
            rp = EclKW("RPORV", grid.getNumActive(), EclDataType.ECL_FLOAT)
            for idx, val in enumerate(rporv):
                rp[idx] = val
            rp.fwrite(f)

        seq_hdr[0] = 2
        header[66] = 2010
        for i, _ in enumerate(p):
            p[i] = 20

        seq_hdr.fwrite(f)
        header.fwrite(f)
        p.fwrite(f)

        if rporv:
            rp = EclKW("RPORV", grid.getNumActive(), EclDataType.ECL_FLOAT)
            for idx, val in enumerate(rporv):
                rp[idx] = val
            rp.fwrite(f)

        seq_hdr[0] = 3
        header[66] = 2011
        for i, _ in enumerate(p):
            p[i] = 25

        seq_hdr.fwrite(f)
        header.fwrite(f)
        p.fwrite(f)

        if rporv:
            rp = EclKW("RPORV", grid.getNumActive(), EclDataType.ECL_FLOAT)
            for idx, val in enumerate(rporv):
                rp[idx] = val
            rp.fwrite(f)


def create_segy_file(name, spec, trace=None, il=None, xl=None, cdp_x=None, cdp_y=None):

    if trace is None:
        trace = np.empty(len(spec.samples), dtype=np.single)
        for i in range(len(spec.samples)):
            trace[i] = 1500 + i

    int_val = [50, 150]
    if il is None:
        il = int_val
    if xl is None:
        xl = int_val
    if cdp_x is None:
        cdp_x = int_val
    if cdp_y is None:
        cdp_y = int_val

    with segyio.create(name, spec) as f:

        scalar = 1
        trno = 0
        for i, i_cdp_x in enumerate(cdp_x):
            for j, j_cdp_y in enumerate(cdp_y):
                f.header[trno] = {
                    TraceField.CDP_Y: j_cdp_y,
                    TraceField.CDP_X: i_cdp_x,
                    TraceField.CROSSLINE_3D: xl[i],
                    TraceField.INLINE_3D: il[j],
                    TraceField.SourceGroupScalar: scalar,
                }
                f.trace[trno] = trace + 0.5 * trno
                trno += 1


def mock_segy(
    ecl_grid,
    relative_resolution_scale=(1.0, 1.0, 1.0),
    relative_size_scale=(1.0, 1.0),
    relative_position_shift=(0, 0),
    segy_filename="vel_vol.segy",
):

    # get the surface top_corners
    nx = ecl_grid.getNX()
    ny = ecl_grid.getNY()
    nz = ecl_grid.getNZ()
    top_corners = np.empty(shape=(nx + 1, ny + 1, 3), dtype=np.float32)
    for i in range(nx + 1):
        for j in range(ny + 1):
            top_corners[i, j] = ecl_grid.getNodePos(i, j, 0)

    # get the bounding box of the surface
    min_x = np.min(top_corners[:, :, 0])
    max_x = np.max(top_corners[:, :, 0])
    min_y = np.min(top_corners[:, :, 1])
    max_y = np.max(top_corners[:, :, 1])

    # we scale and shift wrt the mid-point of the surface
    mid_point = 0.5 * (max_x + min_x), 0.5 * (max_y + min_y)
    extent_x = max_x - min_x
    extent_y = max_y - min_y
    scale_extent_x = extent_x * relative_size_scale[0]
    scale_extent_y = extent_y * relative_size_scale[1]

    # compute scaled and shifted bounding box
    min_x = mid_point[0] - scale_extent_x / 2 + extent_x * relative_position_shift[0]
    max_x = mid_point[0] + scale_extent_x / 2 + extent_x * relative_position_shift[0]
    min_y = mid_point[1] - scale_extent_y / 2 + extent_y * relative_position_shift[1]
    max_y = mid_point[1] + scale_extent_y / 2 + extent_y * relative_position_shift[1]

    # get the new resolution
    nx, ny, nz = tuple(
        np.multiply(relative_resolution_scale, [nx, ny, nz]).astype(np.int32)
    )

    # traces are currently hardcoded going linearly up 1500-4800 samplez into nz steps
    spec = segyio.spec()
    spec.format = 5
    spec.sorting = 2
    spec.samples = range(0, nz * 4, 4)
    spec.ilines = range(ny)
    spec.xlines = range(nx)
    trace = np.linspace(1500, 4800, nz).astype(np.int32)
    xl = np.linspace(min_x, max_x, nx).astype(np.int32)
    il = np.linspace(min_y, max_y, ny).astype(np.int32)
    cdp_x = np.linspace(min_x, max_x, nx).astype(np.int32)
    cdp_y = np.linspace(min_y, max_y, ny).astype(np.int32)

    # for more complex segy volume ie. traces with might use porosity
    # ie. poro = init.iget_named_kw("PORO")
    create_segy_file(
        segy_filename, spec=spec, trace=trace, il=il, xl=xl, cdp_x=cdp_x, cdp_y=cdp_y
    )


if __name__ == "__main__":
    grid_path = os.path.realpath(
        os.path.join(os.path.dirname(__file__), "../../../test-data/norne")
    )
    grid_file = os.path.join(grid_path, "NORNE_ATW2013.EGRID")
    grid = EclGrid(grid_file, apply_mapaxes=True)

    mock_segy(
        grid,
        relative_resolution_scale=(1, 1, 1),
        relative_size_scale=(1, 1),
        relative_position_shift=(0, 0),
        segy_filename="vol_vel.segy",
    )



# -*- coding: utf-8 -*-
import configsuite
import datetime
from pathlib import Path

from configsuite import MetaKeys as MK
from configsuite import types
from copy import deepcopy


@configsuite.validator_msg("List needs to be of size 2")
def _is_length_equal_2(value):
    return len(value) == 2


@configsuite.validator_msg("Only value of 1 or -1 is allowed")
def _is_int_one(value):
    return value == 1 or value == -1


@configsuite.validator_msg("Vintages must contain at least an entry!")
def _min_length(value):
    return len(value) > 1


@configsuite.validator_msg("Eclbase must have an INIT file")
def _init__exists(value):
    return Path(f"{value}.INIT").exists()


@configsuite.validator_msg("Eclbase must have an EGRID file")
def _egrid__exists(value):
    return Path(f"{value}.EGRID").exists()


@configsuite.validator_msg("Eclbase must have a UNRST file")
def _unrst_exists(value):
    return Path(f"{value}.UNRST").exists()


@configsuite.transformation_msg("Converting list of strings to list of dates")
def _str2dates(value):
    value = deepcopy(value)
    dates = []
    for x in value:
        if isinstance(x, str):
            dates.append(datetime.datetime.strptime(x, "%Y-%m-%d").date())
        else:
            dates.append(x)
    return dates


@configsuite.validator_msg("OTS vintage date must be present in .UNRST file!")
def _vintage_present_in_rst(elem, context):
    return elem in context


def build_schema():
    return {
        MK.Type: types.NamedDict,
        MK.Description: "Overburden time shift job parameters",
        MK.Content: {
            "seabed": {
                MK.Type: types.Number,
                MK.Description: "The depth of the seabead in meters.",
            },
            "rfactor": {
                MK.Type: types.Number,
                MK.Description: "Scales the surface displacement between"
                " base_survey and monitor_survey, eg. 20.",
            },
            "above": {
                MK.Type: types.Number,
                MK.Description: "Distance in meters above the reservoir"
                " where shift is calculated. The distance from the "
                "shallowest cell, eg. 100",
            },
            "convention": {
                MK.Type: types.Integer,
                MK.ElementValidators: (_is_int_one,),
                MK.Description: "Positive or negative shift can be either 1 or -1,"
                " where 1 = monitor-base and -1 = base-monitor."
                "The default value is 1.",
                MK.Default: 1,
            },
            "poisson": {
                MK.Type: types.Number,
                MK.Description: "Poisson ratio. Describes the expansion or"
                " contraction of material in ecl_subsidence_eval.",
            },
            "youngs": {
                MK.Type: types.Number,
                MK.Description: "Youngs modulus. The default is 0.",
                MK.Default: 0,
            },
            "output_dir": {
                MK.Type: types.String,
                MK.Description: "Directory(ies) where the shift is written to disk."
                " Post fixed with type of algorithm: ts, ts_simple, dpv and ts_rporv",
            },
            "horizon": {
                MK.Type: types.String,
                MK.Description: "Path to result irap file, the surface mapped to "
                "the velocity grid, with the depth of horizon.",
                MK.Default: None,
                MK.AllowNone: True,
            },
            "eclbase": {
                MK.Type: types.String,
                MK.ElementValidators: (_init__exists, _egrid__exists, _unrst_exists),
                MK.Description: "Path to the Eclipse case.",
            },
            "vintages_export_file": {
                MK.Type: types.String,
                MK.Description: "Path to resulting text file, which contains all "
                "computed vintage pair dates: lines of x, y, z, ts1, ts2, ts3....",
                MK.Default: None,
                MK.AllowNone: True,
            },
            "velocity_model": {
                MK.Type: types.String,
                MK.Description: "Path to segy file containing the velocity field.",
                MK.Default: None,
                MK.AllowNone: True,
            },
            "mapaxes": {
                MK.Type: types.Bool,
                MK.Description: "Mapping axes from the global to local geometry."
                " Can be True or False. If False EclGrid will not "
                "apply transformation to the grid",
            },
            "vintages": {
                MK.Type: types.NamedDict,
                MK.ElementValidators: (_min_length,),
                MK.Description: "Vintage date pairs: date of base and monitor survey.",
                MK.Content: {
                    "ts_simple": {
                        MK.Type: types.List,
                        MK.Description: "Simple TimeShift geertsma algorithm."
                        "It assumes a constant velocity and is fast.",
                        MK.Content: {
                            MK.Item: {
                                MK.Type: types.List,
                                MK.Content: {
                                    MK.Item: {
                                        MK.Type: types.Date,
                                        MK.ContextValidators: (
                                            _vintage_present_in_rst,
                                        ),
                                    }
                                },
                                MK.ElementValidators: (_is_length_equal_2,),
                                MK.LayerTransformation: _str2dates,
                            }
                        },
                    },
                    "ts": {
                        MK.Type: types.List,
                        MK.Description: "TimeShift geertsma algorithm, which "
                        "uses velocity. Very slow.",
                        MK.Content: {
                            MK.Item: {
                                MK.Type: types.List,
                                MK.Content: {
                                    MK.Item: {
                                        MK.Type: types.Date,
                                        MK.ContextValidators: (
                                            _vintage_present_in_rst,
                                        ),
                                    }
                                },
                                MK.ElementValidators: (_is_length_equal_2,),
                                MK.LayerTransformation: _str2dates,
                            }
                        },
                    },
                    "dpv": {
                        MK.Type: types.List,
                        MK.Description: "Delta pressure multiplied by cell volume, "
                        "which is a faster implementation.",
                        MK.Content: {
                            MK.Item: {
                                MK.Type: types.List,
                                MK.Content: {
                                    MK.Item: {
                                        MK.Type: types.Date,
                                        MK.ContextValidators: (
                                            _vintage_present_in_rst,
                                        ),
                                    }
                                },
                                MK.ElementValidators: (_is_length_equal_2,),
                                MK.LayerTransformation: _str2dates,
                            }
                        },
                    },
                    "ts_rporv": {
                        MK.Type: types.List,
                        MK.Description: "Calculates timeshift without using "
                        "velocity. The velocity is only used to get the surface "
                        "on the velocity grid. It uses a change in porevolume "
                        "from Eclipse (RPORV in .UNRST) as input to Geertsma model.",
                        MK.Content: {
                            MK.Item: {
                                MK.Type: types.List,
                                MK.Content: {
                                    MK.Item: {
                                        MK.Type: types.Date,
                                        MK.ContextValidators: (
                                            _vintage_present_in_rst,
                                        ),
                                    }
                                },
                                MK.ElementValidators: (_is_length_equal_2,),
                                MK.LayerTransformation: _str2dates,
                            }
                        },
                    },
                },
            },
        },
    }


def generate_rst_doc():
    _schema = build_schema()
    rst_doc = configsuite.docs.generate(_schema)
    return rst_doc



import pytest
import rstcheck
import pandas as pd

from semeio.workflows.csv_export2 import csv_export2

from tests.jobs.csv_export2 import conftest

test_header = [
    "ENSEMBLE",
    "REAL",
    "DATE",
    "FOPR",
    "X",
    "FLUID_PARAMS:SWL",
    "FLUID_PARAMS:SGCR",
    "A",
    "H",
]

NORNE_VECS = ["FGPT", "FLPT", "FOPT", "FVPT", "FWPT"]


@pytest.mark.skipif(
    conftest.find_available_test_data() is None, reason="no ert-statoil test-data"
)
@pytest.mark.usefixtures("ert_statoil_test_data")
def test_failed_realization_no_summary_file(ert_statoil_test_data):
    path_file = "pathfile_path_to_failed_realizations.txt"
    export_file = "export.txt"
    csv_export2.csv_exporter(
        runpathfile=path_file,
        time_index="monthly",
        outputfile=export_file,
        column_keys="FOPR",
    )
    verifyExportedFile(
        export_file,
        test_header,
        {("iter-1", 1), ("iter-2", 2), ("iter-1", 0), ("iter-1", 2)},
    )


@pytest.mark.skipif(
    conftest.find_available_test_data() is None, reason="no ert-statoil test-data"
)
@pytest.mark.usefixtures("ert_statoil_test_data")
def test_one_iteration(ert_statoil_test_data):
    path_file = "pathfile.txt"
    export_file = "export.txt"
    csv_export2.csv_exporter(
        runpathfile=path_file,
        time_index="monthly",
        outputfile=export_file,
        column_keys="FOPR",
    )
    verifyExportedFile(
        export_file, test_header, {("iter-1", 0), ("iter-1", 1), ("iter-1", 2)}
    )


@pytest.mark.skipif(
    conftest.find_available_test_data() is None, reason="no ert-statoil test-data"
)
@pytest.mark.usefixtures("ert_statoil_test_data")
def test_missing_realization(ert_statoil_test_data):
    path_file = "pathfile2.txt"
    export_file = "export.txt"
    csv_export2.csv_exporter(
        runpathfile=path_file,
        time_index="monthly",
        outputfile=export_file,
        column_keys="FOPR",
    )
    verifyExportedFile(export_file, test_header, {("iter-1", 2), ("iter-1", 0)})


@pytest.mark.skipif(
    conftest.find_available_test_data() is None, reason="no ert-statoil test-data"
)
@pytest.mark.usefixtures("ert_statoil_test_data")
def test_iterations(ert_statoil_test_data):
    path_file = "pathfile3.txt"
    export_file = "export.txt"
    csv_export2.csv_exporter(
        runpathfile=path_file,
        time_index="monthly",
        outputfile=export_file,
        column_keys="FOPR",
    )
    verifyExportedFile(
        export_file,
        test_header,
        {
            ("iter-1", 0),
            ("iter-1", 1),
            ("iter-1", 2),
            ("iter-2", 0),
            ("iter-2", 1),
            ("iter-2", 2),
        },
    )


@pytest.mark.skipif(
    conftest.find_available_test_data() is None, reason="no ert-statoil test-data"
)
@pytest.mark.usefixtures("ert_statoil_test_data")
def test_no_iterations(ert_statoil_test_data):
    path_file = "pathfile1.txt"
    export_file = "export.txt"

    with pytest.raises(KeyError):
        csv_export2.csv_exporter(
            runpathfile=path_file,
            time_index="monthly",
            outputfile=export_file,
            column_keys="FOPR",
        )


@pytest.mark.parametrize("input_rst", [csv_export2.DESCRIPTION, csv_export2.EXAMPLES])
def test_valid_rst(input_rst):
    """
    Check that the documentation passed through the plugin system is
    valid rst
    """
    assert not list(rstcheck.check(input_rst))


@pytest.mark.usefixtures("norne_mocked_ensembleset")
def test_norne_ensemble(norne_mocked_ensembleset):
    csv_export2.csv_exporter(
        runpathfile="runpathfile",
        time_index="yearly",
        outputfile="unsmry--yearly.csv",
        column_keys=["F?PT"],
    )
    verifyExportedFile(
        "unsmry--yearly.csv",
        ["ENSEMBLE", "REAL", "DATE"] + NORNE_VECS + ["FOO"],
        {
            ("iter-0", 0),
            ("iter-0", 1),
            ("iter-1", 0),
            ("iter-1", 1),
        },
    )


@pytest.mark.usefixtures("norne_mocked_ensembleset_noparams")
def test_norne_ensemble_noparams(norne_mocked_ensembleset_noparams):
    csv_export2.csv_exporter(
        runpathfile="runpathfile",
        time_index="yearly",
        outputfile="unsmry--yearly.csv",
        column_keys=["FOPT"],
    )
    verifyExportedFile(
        "unsmry--yearly.csv",
        ["ENSEMBLE", "REAL", "DATE", "FOPT"],
        {
            ("iter-0", 0),
            ("iter-0", 1),
            ("iter-1", 0),
            ("iter-1", 1),
        },
    )


def verifyExportedFile(exported_file_name, result_header, result_iter_rel):
    """Verify an exported CSV file with respect to:

        * Exactly the set of requested headers is found
        * The realizations and iterations that exist must equal
          given set of tuples.

    Args:
        exported_file_name (str): path to CSV file.
        result_header (list of str): The strings required in the header.
        result_iter_real (set): Set of 2-tuples: {(iterstring, realidx)}
    """
    dframe = pd.read_csv(exported_file_name)
    assert set(dframe.columns) == set(result_header)
    assert (
        set(dframe[["ENSEMBLE", "REAL"]].itertuples(index=False, name=None))
        == result_iter_rel
    )



#!/usr/bin/env python

import json


def _load_coeffs(filename):
    with open(filename) as f:
        return json.load(f)


def _evaluate(coeffs, x):
    return coeffs["a"] * x ** 2 + coeffs["b"] * x + coeffs["c"]


if __name__ == "__main__":
    coeffs = _load_coeffs("coeffs.json")
    output = [_evaluate(coeffs, x) for x in range(10)]
    with open("poly_0.out", "w") as f:
        f.write("\n".join(map(str, output)))



#!/usr/bin/env python
import sys

iens = None
if len(sys.argv) > 1:
    iens = int(sys.argv[1])

if iens is None or iens not in [2, 3, 5, 7]:
    with open("realization.ok", "w") as f:
        f.write("OK")



import logging
from collections import namedtuple
import os.path
from datetime import datetime as dt
from pathlib import Path

import numpy as np
from scipy.interpolate import CloughTocher2DInterpolator

from ecl.grid import EclGrid
from ecl.gravimetry import EclSubsidence
from ecl.eclfile import Ecl3DKW, EclFile

from ecl.util.geometry import Surface

from semeio.jobs.overburden_timeshift.ots_vel_surface import OTSVelSurface
from semeio.jobs.overburden_timeshift.ots_res_surface import OTSResSurface
from semeio.jobs.overburden_timeshift.ots_config import build_schema
from semeio._exceptions.exceptions import ConfigurationError
import configsuite
import yaml


# pylint: disable=consider-using-enumerate
def extract_ots_context(configuration):
    rstfile_path = Path(f"{configuration.eclbase}.UNRST")
    if not rstfile_path.exists():
        return []
    dates = [d.date() for d in EclFile(str(rstfile_path)).dates]
    return dates


def ots_load_params(input_file):
    with open(input_file, "r") as fin:
        config = yaml.safe_load(fin)
    config = configsuite.ConfigSuite(
        config,
        build_schema(),
        deduce_required=True,
        extract_validation_context=extract_ots_context,
    )
    if not config.valid:
        raise ConfigurationError(
            "Invalid configuration for config file: {}".format(input_file),
            config.errors,
        )

    input_data = config.snapshot
    return input_data


def write_surface(vintage_pairs, ts, output_dir, type_str):
    for iv, vp in enumerate(vintage_pairs):
        d0 = vp[0].strftime("%Y_%m_%d")
        d1 = vp[1].strftime("%Y_%m_%d")
        ts[iv].write(output_dir + type_str + "/ots" + "_" + d0 + "_" + d1 + ".irap")


def ots_run(parameter_file):

    parms = ots_load_params(parameter_file)
    vintage_pairs = parms.vintages

    ots = OverburdenTimeshift(
        eclbase=parms.eclbase,
        mapaxes=parms.mapaxes,
        seabed=parms.seabed,
        youngs=parms.youngs,
        poisson=parms.poisson,
        rfactor=parms.rfactor,
        convention=parms.convention,
        above=parms.above,
        velocity_model=parms.velocity_model,
    )

    tshift_ts = ots.geertsma_ts(vintage_pairs.ts)
    tshift_ts_simple = ots.geertsma_ts_simple(vintage_pairs.ts_simple)
    tshift_dpv = ots.dpv(vintage_pairs.dpv)
    tshift_ts_rporv = ots.geertsma_ts_rporv(vintage_pairs.ts_rporv)

    surface_horizon = ots.get_horizon()

    write_surface(vintage_pairs.ts, tshift_ts, parms.output_dir, "_ts")
    write_surface(
        vintage_pairs.ts_simple, tshift_ts_simple, parms.output_dir, "_ts_simple"
    )
    write_surface(vintage_pairs.dpv, tshift_dpv, parms.output_dir, "_dpv")
    write_surface(
        vintage_pairs.ts_rporv, tshift_ts_rporv, parms.output_dir, "_ts_rporv"
    )

    if parms.horizon is not None:
        surface_horizon.write(parms.horizon)

    if parms.vintages_export_file is not None:
        num_pairs = (
            len(vintage_pairs.ts)
            + len(vintage_pairs.ts_simple)
            + len(vintage_pairs.dpv)
            + len(vintage_pairs.ts_rporv)
        )
        line = "{}, {}, {}" + ", {}" * num_pairs + "\n"
        with open(parms.vintages_export_file, "w") as f:
            for point in range(len(surface_horizon)):
                xy = surface_horizon.getXY(point)
                ts = []
                for iv in range(len(vintage_pairs.ts)):
                    ts.append(tshift_ts[iv][point])
                for iv in range(len(vintage_pairs.ts_simple)):
                    ts.append(tshift_ts_simple[iv][point])
                for iv in range(len(vintage_pairs.dpv)):
                    ts.append(tshift_dpv[iv][point])
                for iv in range(len(vintage_pairs.ts_rporv)):
                    ts.append(tshift_ts_rporv[iv][point])
                f.write(line.format(xy[0], xy[1], surface_horizon[point], *ts))


class OverburdenTimeshift(object):
    def __init__(
        self,
        eclbase,
        mapaxes,
        seabed,
        youngs,
        poisson,
        rfactor,
        convention,
        above,
        velocity_model,
    ):
        """
        The OTS class manages the information required to calculate
        overburden timeshift.

        The constructor will look for the Eclipse files INIT, EGRID
        and UNRST based on the input case, if some of the files are
        missing an exception will be raised. It will then instantiate
        a EclSubsidence object will be used to manage the rest of the
        overburden timeshift calculations.
        """
        case = os.path.splitext(eclbase)[0]
        self._init_file = EclFile("%s.INIT" % case)
        self._rst_file = EclFile("%s.UNRST" % case)
        self._grid = EclGrid("%s.EGRID" % case, apply_mapaxes=mapaxes)

        self.subsidence = EclSubsidence(self._grid, self._init_file)

        self._seabed = seabed
        self._youngs_modulus = youngs * 1e9
        self._poisson_ratio = poisson
        self._r_factor = rfactor
        self._convention = convention

        self._surface = res_surface = OTSResSurface(grid=self._grid, above=above)
        if velocity_model is not None:
            self._surface = OTSVelSurface(res_surface=res_surface, vcube=velocity_model)

        self._restart_views = {}

    def get_horizon(self):
        return self._create_surface()

    def _create_surface(self, z=None):
        """
        Generate irap surface

        :param z: replace z values of surface
        """
        nx = self._surface.nx
        ny = self._surface.ny
        x = self._surface.x
        y = self._surface.y
        if z is None:
            z = self._surface.z

        xstart = np.min(x)
        ystart = np.min(y)

        if nx < 2 or ny < 2:
            raise RuntimeError("Cannot create IRAP surface if nx or ny is <2")

        xinc = (np.max(x) - xstart) / (nx - 1)
        yinc = (np.max(y) - ystart) / (ny - 1)

        surf = Surface(
            nx=nx, ny=ny, xinc=xinc, yinc=yinc, xstart=xstart, ystart=ystart, angle=0
        )

        irap_x = np.empty(nx * ny)
        irap_y = np.array(irap_x)

        for i in range(len(surf)):
            irap_x[i], irap_y[i] = surf.getXY(i)

        # Interpolate vel grid to irap grid, should be the same apart from ordering
        z = np.nan_to_num(z)
        ip = CloughTocher2DInterpolator((x, y), z, fill_value=0)
        irap_z = ip(irap_x, irap_y)

        for i in range(len(surf)):
            surf[i] = irap_z[i]

        return surf

    def add_survey(self, name, date):
        """The add_survey() method will register a survey at a specific date.

        The name argument should be a unique string, this will later
        be used when evaluating the elastic strain in the
        overburden. The date should be python datetime.date()
        instance, this date should be present as a report step in the
        restart file - otherwise an exception will be raised.
        """
        restart_view = self._rst_file.restartView(sim_time=date)
        self.subsidence.add_survey_PRESSURE(name, restart_view)

        self._restart_views[name] = restart_view

        return restart_view

    @staticmethod
    def _divide_negative_shift(ts, div_val=5.0):
        for i in range(len(ts)):
            if ts[i] < 0:
                ts[i] /= div_val

    def geertsma_ts_rporv(self, vintage_pairs):
        """
        Calculates TS without using velocity. Fast.
        Velocity is only used to get the surface on the velocity grid.
        Uses change in porevolume from Eclipse (RPORV in .UNRST) as input to
        Geertsma model.

        :param vintage_pairs:
        """
        return self._geertsma_ts_custom(
            vintage_pairs, self.subsidence.eval_geertsma_rporv, "TS_RPORV"
        )

    def geertsma_ts_simple(self, vintage_pairs):
        """
        Calculates TS without using velocity. Fast.
        Velocity is only used to get the surface on the velocity grid.

        :param vintage_pairs:
        """
        return self._geertsma_ts_custom(
            vintage_pairs, self.subsidence.evalGeertsma, "TS_SIMPLE"
        )

    def _geertsma_ts_custom(self, vintage_pairs, subsidence_func, method_name):
        """
        Calculates TS without using velocity. Fast.

        :param vintage_pairs:
        :param subsidence_func: specify subsidence method to be used
        :param method_name: string represeting the subsudence func name
        """

        if len(vintage_pairs) < 1:
            return 0, []

        vintages = self._vintages_name_date(vintage_pairs)
        surface = self._surface
        points_to_calculate = self._get_non_nan_points()
        surf_displacement = {}
        ts_surfaces = []
        for vintage in vintages:
            logging.info(
                "{:%x %X} {}: Calculating vintage {:%Y.%m.%d}".format(
                    dt.now(), method_name, vintage.date
                )
            )

            self.add_survey(vintage.name, vintage.date)
            surf_displacement[vintage.date] = np.zeros(len(surface))

            for point in points_to_calculate:
                r1 = (surface.x[point], surface.y[point], 0)
                r2 = (
                    surface.x[point],
                    surface.y[point],
                    surface.z[point] - self._seabed,
                )
                # subsidence and displacement have opposite sign
                # should have minus on dz1 and dz2 here, more efficient
                # when calculating ts
                dz1 = subsidence_func(
                    base_survey=vintage.name,
                    monitor_survey=None,
                    pos=r1,
                    youngs_modulus=self._youngs_modulus,
                    poisson_ratio=self._poisson_ratio,
                    seabed=self._seabed,
                )
                dz2 = subsidence_func(
                    base_survey=vintage.name,
                    monitor_survey=None,
                    pos=r2,
                    youngs_modulus=self._youngs_modulus,
                    poisson_ratio=self._poisson_ratio,
                    seabed=self._seabed,
                )
                surf_displacement[vintage.date][point] = dz2 - dz1

        for base, monitor in vintage_pairs:
            self._report(method_name, base, monitor, len(points_to_calculate))

            ts = -self._r_factor * (
                surf_displacement[monitor] - surf_displacement[base]
            )
            # Observations (even though they are few) indicate that time shifts
            # linked with compaction are smaller than when linked with stretch.
            # This is also reported by e.g. Hatchel and Bourne (2005)
            # Approximatelly R_comp = R_stretch / 5 was chosen
            self._divide_negative_shift(ts, div_val=5.0)

            ts = ts * self._convention

            ts_surfaces.append(self._create_surface(ts))

        return ts_surfaces

    def geertsma_ts(self, vintage_pairs):
        """
        Calculates TS using velocity. Slow.

        :param vintage_pairs:
        """

        if len(vintage_pairs) < 1:
            return 0, []

        surface = self._surface
        points_to_calculate = self._get_non_nan_points()

        ts_surfaces = []
        _, nz = surface.z3d.shape

        for base, monitor in vintage_pairs:
            surf_displacement = np.zeros(len(surface))
            logging.info(
                "{:%x %X} TS: Calculating vintage"
                " {:%Y.%m.%d} - {:%Y.%m.%d}".format(dt.now(), base, monitor)
            )

            # According to the author, base and monitor are
            # reversed for geertsma_ts
            self.add_survey("base", monitor)
            self.add_survey("monitor", base)

            for point in points_to_calculate:
                for iz in range(nz):
                    rz = surface.z3d[point, iz] - self._seabed
                    if 0 <= rz <= (surface.z[point] - self._seabed):
                        r1 = (surface.x[point], surface.y[point], rz)
                        r2 = (surface.x[point], surface.y[point], rz + 0.1)
                        # subsidence and displacement have opposite sign
                        # should have minus here, more efficient
                        # when calculating ts
                        dz1 = self.subsidence.evalGeertsma(
                            base_survey="base",
                            monitor_survey="monitor",
                            pos=r1,
                            youngs_modulus=self._youngs_modulus,
                            poisson_ratio=self._poisson_ratio,
                            seabed=self._seabed,
                        )
                        dz2 = self.subsidence.evalGeertsma(
                            base_survey="base",
                            monitor_survey="monitor",
                            pos=r2,
                            youngs_modulus=self._youngs_modulus,
                            poisson_ratio=self._poisson_ratio,
                            seabed=self._seabed,
                        )

                        # The vertical strain is equal to dz/z, where dz is
                        # the thickness change within a chosen thickness z.
                        verical_strain = (dz2 - dz1) * 10
                        surf_displacement[point] += verical_strain

            self._report("TS", base, monitor, len(points_to_calculate))

            # subsidence and displacement have opposite sign, thus the minus sign
            ts = -self._r_factor * surf_displacement * surface.dt * 1000
            self._divide_negative_shift(ts)

            ts = ts * self._convention

            ts_surfaces.append(self._create_surface(ts))

        return ts_surfaces

    @staticmethod
    def _vintages_name_date(vintage_pairs):
        vintages = set()
        for pair in vintage_pairs:
            vintages.add(pair[0])
            vintages.add(pair[1])
        vintages_date = list(vintages)
        vintages_name = []
        for i in range(len(vintages_date)):
            vintages_name.append("S{}".format(i))

        Vintage = namedtuple("Vintages", "name date")
        return [Vintage(name, date) for name, date in zip(vintages_name, vintages_date)]

    def _report(self, func_name, base, monitor, num_points_calculated):
        if self._convention == 1:
            start_date, end_date = monitor, base
        elif self._convention == -1:
            start_date, end_date = base, monitor
        else:
            raise ValueError(
                "Convention must be 1 or -1, was {}".format(self._convention)
            )
        logging.debug(
            "{:%x %X} {}: Calculating shift"
            " {:%Y.%m.%d}-{:%Y.%m.%d} in {} points".format(
                dt.now(), func_name, start_date, end_date, num_points_calculated
            )
        )

    def _get_non_nan_points(self):
        points_to_calculate = [
            _id
            for _id in range(len(self._surface))
            if not np.isnan(self._surface.z[_id])
        ]
        if len(points_to_calculate) == 0:
            logging.error(
                (
                    "Overburden timeshift was calculated in 0 points. ",
                    "Consider changing --apply_mapaxes value.",
                )
            )
        return points_to_calculate

    def dpv(self, vintage_pairs):
        """
        Calulates change in pressure multiplied by cell volume
        and sum for all cells in column

        dPV must have equal sign as TS, but opposite from mathematics

        monitor-base

        :param vintage_pairs: list of pairs of vintages
        :return:
        """

        if len(vintage_pairs) < 1:
            return 0, []

        vintages = self._vintages_name_date(vintage_pairs)

        surf = self._surface
        points_to_calculate = self._get_non_nan_points()

        shift_surfaces = []
        pressure_volume = {}

        for vintage in vintages:
            logging.info(
                "{:%x %X} DPV: Calculating vintage"
                " {:%Y.%m.%d}".format(dt.now(), vintage.date)
            )
            self.add_survey(vintage.name, vintage.date)
            pressure = Ecl3DKW.castFromKW(
                self._restart_views[vintage.name]["PRESSURE"][0], self._grid
            )
            pressure_volume[vintage.date] = np.zeros(len(surf))

            for point in points_to_calculate:
                r = surf.x[point], surf.y[point], 0
                try:
                    i, j = self._grid.findCellXY(*r)
                    sum_pv = 0
                    for k in range(self._grid.getNZ()):
                        v = self._grid.cell_volume(ijk=(i, j, k))
                        sum_pv += pressure[i, j, k] * v

                    pressure_volume[vintage.date][point] = sum_pv
                except ValueError:
                    pressure_volume[vintage.date][point] = 0

        for base, monitor in vintage_pairs:
            self._report("DPV", base, monitor, len(points_to_calculate))
            dpv = (
                (pressure_volume[monitor] - pressure_volume[base])
                / 1e9
                * self._convention
            )
            shift_surfaces.append(self._create_surface(dpv))

        return shift_surfaces



class ConfigurationError(Exception):
    def __init__(self, message, errors):
        super().__init__(message)
        self.errors = errors
        self.message = message

    def __str__(self):
        msg = "{}".format(self.message)
        for error in self.errors:
            msg += "\n{}".format(error)
        return msg


class ValidationError(Exception):
    pass



from res.enkf import EnKFMain
from semeio.workflows.correlated_observations_scaling.update_scaling import (
    scale_observations,
)
import pytest


class Config:  # pylint: disable=too-few-public-methods
    def __init__(self, key, index):
        self.key = key
        self.index = index


@pytest.fixture()
def snake_oil_obs(setup_ert):
    res_config = setup_ert
    ert = EnKFMain(res_config)
    yield ert.getObservations()


@pytest.mark.parametrize("index_list", [None, [0, 1, 2, 3]])
def test_scale_history_summary_obs(snake_oil_obs, index_list):

    scale_observations(snake_oil_obs, 1.2345, [Config("FOPR", index_list)])

    obs_vector = snake_oil_obs["FOPR"]
    for index, node in enumerate(obs_vector):
        if not index_list or index in index_list:
            assert node.getStdScaling(index) == 1.2345, f"index: {index}"
        else:
            assert node.getStdScaling(index) == 1.0, f"index: {index}"


@pytest.mark.parametrize("index_list", [None, [35]])
def test_scale_summary_obs(snake_oil_obs, index_list):

    scale_observations(snake_oil_obs, 1.2345, [Config("WOPR_OP1_36", index_list)])

    obs_vector = snake_oil_obs["WOPR_OP1_36"]
    node = obs_vector.getNode(36)
    assert node.getStdScaling(36) == 1.2345, f"index: {36}"


@pytest.mark.parametrize("index_list", [None, [400, 800]])
def test_scale_gen_obs(snake_oil_obs, index_list):

    scale_observations(snake_oil_obs, 1.2345, [Config("WPR_DIFF_1", index_list)])

    obs_vector = snake_oil_obs["WPR_DIFF_1"]
    for index, node in enumerate(obs_vector):
        if not index_list or node.getDataIndex(index) in index_list:
            assert node.getStdScaling(index) == 1.2345, f"index: {index}"
        else:
            assert node.getStdScaling(index) == 1.0, f"index: {index}"



import sys
from semeio._exceptions.exceptions import ConfigurationError
from semeio.jobs.overburden_timeshift.ots import ots_run
from semeio.jobs.overburden_timeshift.ots_config import generate_rst_doc
import argparse
from semeio import valid_file

short_description = (
    "Overburden timeshift (OTS) generates evolution of reservoir surfaces "
    "based on eclipse models and seismic velocity volume. If the shift "
    "becomes negative it won't be nullified but only divided "
    "by a constant (currently defaults to 5), ie. Time shifts "
    "linked with compaction are smaller than those linked with stretch. "
    "Input yml needs to contains vintages section, where at "
    "least one of the four categories "
    "for the surface computation needs to be set. "
    "By specifying --params, one gets the full list of yml config parameters."
)

description = short_description + generate_rst_doc()
category = "modeling.surface"


def _get_args_parser():
    parser = argparse.ArgumentParser(description=short_description)

    parser.add_argument(
        "-c", "--config", help="ots config file, yaml format required", type=valid_file
    )
    parser.add_argument(
        "-p",
        "--params",
        action="store_true",
        help="shows all ots config job parameters",
        dest="help_params",
        default=False,
    )
    return parser


def main_entry_point():
    parser = _get_args_parser()
    options = parser.parse_args()
    if options.help_params:
        print(generate_rst_doc())
    else:
        try:
            ots_run(options.config)
        except ConfigurationError as err:
            sys.exit(str(err))



import os
import gc
import pytest
from tests import legacy_test_data


@pytest.fixture()
def test_data_root():
    yield legacy_test_data.__path__[0]


@pytest.fixture()
def setup_tmpdir(tmpdir):
    with tmpdir.as_cwd():
        yield
    # This makes sure EnkFMain is cleaned up between
    # each test, otherwise a storage folder is created
    # in semeio root dir at the end of the test run.
    # Should be deleted when we no longer use EnkFMain
    gc.collect()


@pytest.fixture(scope="session", autouse=True)
def move_to_shared_tmp(tmp_path_factory):
    """
    We do this because when running some of the tests individually
    they create a storage folder in the semeio root directory during
    tear down. This places that storage folder in a shared tempdir
    instead. Should be deleted when we no longer use EnkFMain
    """
    cwd = os.getcwd()
    os.chdir(tmp_path_factory.mktemp("some_shared_path"))
    yield
    os.chdir(cwd)



import logging

from semeio.workflows.correlated_observations_scaling.obs_utils import (
    create_active_lists,
)


def scale_observations(obs, scale_factor, obs_list):
    """
    Function to scale observations
    :param obs: enkfObservations type
    :param scale_factor: float
    :param obs_list: list of observations to scale, must have fields key and index
    :return: None, scales observations in place
    """
    update_data = create_active_lists(obs, obs_list)
    _update_scaling(obs, scale_factor, update_data)


def _update_scaling(obs, scale_factor, obs_list):
    """
    Applies the scaling factor to the user specified index, SUMMARY_OBS needs to be
    treated differently as it only has one data point per node, compared with other
    observation types which have multiple data points per node.
    """
    for event in obs_list:
        obs_vector = obs[event.key]
        step_list = list(obs_vector.getStepList())  # List of steps, 1-indexed
        for step in step_list:
            obs_node = obs_vector.getNode(step)
            if obs_vector.getImplementationType().name == "SUMMARY_OBS":
                index_list = event.index if event.index else [x - 1 for x in step_list]
                if step - 1 in index_list:
                    obs_node.set_std_scaling(scale_factor)
            else:
                obs_node.updateStdScaling(scale_factor, event.active_list)
    logging.info(
        "Keys: {} scaled with scaling factor: {}".format(
            [event.key for event in obs_list], scale_factor
        )
    )



import os
import pytest
import datetime
from collections import namedtuple

from ecl.grid import EclGridGenerator
from ecl.util.geometry import Surface
import segyio

from semeio.jobs.overburden_timeshift.ots import OverburdenTimeshift
from ots_util import create_init, create_restart, create_segy_file

parms = namedtuple(
    "Parms",
    [
        "seabed",
        "above",
        "youngs",
        "poisson",
        "rfactor",
        "mapaxes",
        "convention",
        "output_dir",
        "horizon",
        "vintages_export_file",
        "velocity_model",
        "eclbase",
    ],
)


@pytest.fixture()
def set_up():
    spec = segyio.spec()
    spec.format = 5
    spec.sorting = 2
    spec.samples = range(0, 40, 4)
    spec.ilines = range(2)
    spec.xlines = range(2)

    actnum = [0, 0, 0, 0, 1, 1, 1, 1]

    parms.output_dir = None
    parms.horizon = None
    parms.vintages_export_file = None
    parms.velocity_model = None
    parms.seabed = 10
    parms.above = 10
    parms.youngs = 0.5
    parms.poisson = 0.3
    parms.rfactor = 20
    parms.convention = 1
    parms.eclbase = "TEST"

    yield spec, actnum, parms


@pytest.mark.parametrize(
    "missing_file, expected_error",
    [
        ("TEST.INIT", 'Failed to open file "TEST.INIT"'),
        ("TEST.EGRID", "Loading grid from:TEST.EGRID failed"),
        ("TEST.UNRST", 'Failed to open file "TEST.UNRST"'),
    ],
)
@pytest.mark.usefixtures("setup_tmpdir")
def test_create_missing_ecl_file(set_up, missing_file, expected_error):
    spec, actnum, params = set_up
    grid = EclGridGenerator.createRectangular(dims=(10, 10, 10), dV=(1, 1, 1))

    grid.save_EGRID("TEST.EGRID")
    create_init(grid, "TEST")
    create_restart(grid, "TEST")

    os.remove(missing_file)
    with pytest.raises(IOError, match=expected_error):
        OverburdenTimeshift(
            params.eclbase,
            params.mapaxes,
            params.seabed,
            params.youngs,
            params.poisson,
            params.rfactor,
            params.convention,
            params.above,
            params.velocity_model,
        )


def test_create_invalid_input_missing_segy(set_up):
    spec, actnum, parms = set_up

    grid = EclGridGenerator.createRectangular(dims=(10, 10, 10), dV=(1, 1, 1))
    grid.save_EGRID("TEST.EGRID")

    create_init(grid, "TEST")
    create_restart(grid, "TEST")

    parms.velocity_model = "missing.segy"

    with pytest.raises(IOError, match="No such file or directory"):
        OverburdenTimeshift(
            parms.eclbase,
            parms.mapaxes,
            parms.seabed,
            parms.youngs,
            parms.poisson,
            parms.rfactor,
            parms.convention,
            parms.above,
            parms.velocity_model,
        )


@pytest.mark.parametrize(
    "config_item, value", [("velocity_model", "TEST.segy"), ("velocity_model", None)]
)
@pytest.mark.usefixtures("setup_tmpdir")
def test_create_valid(set_up, config_item, value):
    spec, actnum, params = set_up
    grid = EclGridGenerator.createRectangular(dims=(10, 10, 10), dV=(1, 1, 1))

    grid.save_EGRID("TEST.EGRID")
    create_init(grid, "TEST")
    create_restart(grid, "TEST")

    # Testing individual items in config, so setting that value here:
    setattr(params, config_item, value)
    if params.velocity_model:
        create_segy_file(params.velocity_model, spec)

    OverburdenTimeshift(
        params.eclbase,
        params.mapaxes,
        params.seabed,
        params.youngs,
        params.poisson,
        params.rfactor,
        params.convention,
        params.above,
        params.velocity_model,
    )


@pytest.mark.usefixtures("setup_tmpdir")
def test_eval(set_up):
    spec, actnum, parms = set_up
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=actnum
    )

    grid.save_EGRID("TEST.EGRID")
    create_init(grid, "TEST")
    create_restart(grid, "TEST")

    parms.velocity_model = "TEST.segy"
    create_segy_file(parms.velocity_model, spec)

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )
    with pytest.raises(ValueError):
        ots.add_survey("S1", datetime.date(2000, 1, 15))

    vintage_pairs = [(datetime.date(1900, 1, 1), datetime.date(2010, 1, 1))]

    with pytest.raises(ValueError):
        ots.geertsma_ts_simple(vintage_pairs)

    vintage_pairs = [(datetime.date(2010, 1, 1), datetime.date(1900, 1, 1))]

    with pytest.raises(ValueError):
        ots.geertsma_ts_simple(vintage_pairs)

    vintage_pairs = [(datetime.date(2000, 1, 1), datetime.date(2010, 1, 1))]

    ots.geertsma_ts_simple(vintage_pairs)


@pytest.mark.usefixtures("setup_tmpdir")
def test_geertsma_TS_simple(set_up):
    spec, actnum, parms = set_up
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=actnum
    )

    create_restart(grid, "TEST")
    create_init(grid, "TEST")
    grid.save_EGRID("TEST.EGRID")

    parms.velocity_model = "TEST.segy"

    int_val = [50, 150]
    create_segy_file(
        parms.velocity_model, spec, xl=int_val, il=int_val, cdp_x=int_val, cdp_y=int_val
    )

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    vintage_pairs = [
        (datetime.date(2000, 1, 1), datetime.date(2010, 1, 1)),
        (datetime.date(2010, 1, 1), datetime.date(2011, 1, 1)),
    ]

    tshift = ots.geertsma_ts_simple(vintage_pairs)
    assert tshift[0][0] == pytest.approx(-0.01006, abs=0.0001)

    parms.convention = -1
    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    vintage_pairs = [
        (datetime.date(2000, 1, 1), datetime.date(2010, 1, 1)),
        (datetime.date(2010, 1, 1), datetime.date(2011, 1, 1)),
    ]

    tshift = ots.geertsma_ts_simple(vintage_pairs)
    assert tshift[0][0] == pytest.approx(0.01006, abs=0.0001)


@pytest.mark.usefixtures("setup_tmpdir")
def test_geertsma_TS_rporv(set_up):
    spec, actnum, parms = set_up
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=actnum
    )

    create_restart(grid, "TEST", rporv=[10 for i in range(grid.getNumActive())])
    create_init(grid, "TEST")
    grid.save_EGRID("TEST.EGRID")

    parms.velocity_model = "TEST.segy"

    int_val = [50, 150]
    create_segy_file(
        parms.velocity_model, spec, xl=int_val, il=int_val, cdp_x=int_val, cdp_y=int_val
    )

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    vintage_pairs = [
        (datetime.date(2000, 1, 1), datetime.date(2010, 1, 1)),
        (datetime.date(2010, 1, 1), datetime.date(2011, 1, 1)),
    ]

    tshift = ots.geertsma_ts_rporv(vintage_pairs)
    assert tshift[0][0] == pytest.approx(0.0, abs=0.0001)


@pytest.mark.usefixtures("setup_tmpdir")
def test_geertsma_TS(set_up):
    spec, actnum, parms = set_up
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=actnum
    )

    create_restart(grid, "TEST")
    create_init(grid, "TEST")
    grid.save_EGRID("TEST.EGRID")

    parms.velocity_model = "TEST.segy"

    int_val = [50, 150]
    create_segy_file(
        parms.velocity_model, spec, xl=int_val, il=int_val, cdp_x=int_val, cdp_y=int_val
    )

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    vintage_pairs = [
        (datetime.date(2000, 1, 1), datetime.date(2010, 1, 1)),
        (datetime.date(2010, 1, 1), datetime.date(2011, 1, 1)),
    ]

    tshift = ots.geertsma_ts(vintage_pairs)

    assert tshift[0][0] == pytest.approx(-0.00104, abs=0.0001)

    parms.convention = -1

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    vintage_pairs = [
        (datetime.date(2000, 1, 1), datetime.date(2010, 1, 1)),
        (datetime.date(2010, 1, 1), datetime.date(2011, 1, 1)),
    ]

    tshift = ots.geertsma_ts(vintage_pairs)
    assert tshift[0][0] == pytest.approx(0.00104, abs=0.0001)


@pytest.mark.usefixtures("setup_tmpdir")
def test_dPV(set_up):
    spec, actnum, parms = set_up
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=actnum
    )

    grid.save_EGRID("TEST.EGRID")
    create_restart(grid, "TEST")
    create_init(grid, "TEST")

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    vintage_pairs = [
        (datetime.date(2000, 1, 1), datetime.date(2010, 1, 1)),
        (datetime.date(2010, 1, 1), datetime.date(2011, 1, 1)),
    ]

    tshift = ots.dpv(vintage_pairs)
    assert tshift[0][0] == pytest.approx(((20 - 10) * 1e6 + (0 - 0) * 1e6) / 1e9)
    assert tshift[0][2] == pytest.approx(((20 - 10) * 1e6 + (0 - 0) * 1e6) / 1e9)

    assert tshift[1][0] == pytest.approx(((25 - 20) * 1e6 + (0 - 0) * 1e6) / 1e9)
    assert tshift[1][2] == pytest.approx(((25 - 20) * 1e6 + (0 - 0) * 1e6) / 1e9)

    parms.convention = -1

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )
    tshift_b_m = ots.dpv(vintage_pairs)
    assert tshift[0][0] == pytest.approx(-tshift_b_m[0][0])


@pytest.mark.usefixtures("setup_tmpdir")
def test_irap_surface(set_up):
    spec, actnum, parms = set_up
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=actnum
    )

    # with TestAreaContext("test_irap_surface"):
    create_restart(grid, "TEST")
    create_init(grid, "TEST")
    grid.save_EGRID("TEST.EGRID")

    parms.velocity_model = "TEST.segy"
    create_segy_file(parms.velocity_model, spec)

    ots = OverburdenTimeshift(
        parms.eclbase,
        parms.mapaxes,
        parms.seabed,
        parms.youngs,
        parms.poisson,
        parms.rfactor,
        parms.convention,
        parms.above,
        parms.velocity_model,
    )

    f_name = "irap.txt"
    s = ots._create_surface()
    s.write(f_name)
    s = Surface(f_name)

    assert s.getNX() == 2
    assert s.getNY() == 2

    for val in s:
        assert val == 90



import os
import shutil

import pytest

TEST_DATA_DIR = os.environ.get("ERT_TESTDATA_ROOT")
TEST_DATA_DIR_BERGEN = "/d/proj/bg/enkf/ErtTestData"
TEST_DATA_DIR_STAVANGER = "/project/res-testdata/ErtTestData"

NORNE_DIR = os.path.join(os.path.dirname(__file__), "../../test_data/norne")


def find_available_test_data():
    if TEST_DATA_DIR is None:
        if os.path.isdir(TEST_DATA_DIR_STAVANGER):
            return TEST_DATA_DIR_STAVANGER
        elif os.path.isdir(TEST_DATA_DIR_BERGEN):
            return TEST_DATA_DIR_BERGEN

    return TEST_DATA_DIR


def has_equinor_test_data():
    return os.path.isdir(find_available_test_data())


def pytest_runtest_setup(item):
    if item.get_closest_marker("equinor_test") and not has_equinor_test_data():
        pytest.skip("Test requires Equinor data")


@pytest.fixture()
def ert_statoil_test_data(tmpdir):
    cwd = os.getcwd()
    tmpdir.chdir()
    test_data_dir = os.path.join(find_available_test_data(), "ert-statoil", "spotfire")
    shutil.copytree(test_data_dir, "test_data")
    os.chdir(os.path.join("test_data"))

    yield

    os.chdir(cwd)


def mock_norne_data(reals, iters, parameters=True):
    """From a single UNSMRY file, produce arbitrary sized ensembles.

    Summary data will be equivalent over realizations, but the
    parameters.txt is made unique.

    Writes realization-*/iter-* file structure in cwd.

    Args:
        reals (list): integers with realization indices wanted
        iters (list): integers with iter indices wanted
        parameters (bool): Whether to write parameters.txt in each runpath
    """
    for real in reals:
        for iteration in iters:
            runpath = os.path.join(
                "realization-{}".format(real), "iter-{}".format(iteration)
            )

            os.makedirs(runpath, exist_ok=True)

            os.symlink(
                os.path.join(NORNE_DIR, "NORNE_ATW2013.UNSMRY"),
                os.path.join(runpath, "NORNE_{}.UNSMRY".format(real)),
            )
            os.symlink(
                os.path.join(NORNE_DIR, "NORNE_ATW2013.SMSPEC"),
                os.path.join(runpath, "NORNE_{}.SMSPEC".format(real)),
            )
            if parameters:
                with open(os.path.join(runpath, "parameters.txt"), "w") as p_fileh:
                    p_fileh.write("FOO 1{}{}".format(real, iteration))

    with open("runpathfile", "w") as file_h:
        for iteration in iters:
            for real in reals:
                runpath = os.path.join(
                    "realization-{}".format(real), "iter-{}".format(iteration)
                )
                file_h.write(
                    "{0:03d} {1} NORNE_{0} {2:03d}\n".format(real, runpath, iteration)
                )


@pytest.fixture()
def norne_mocked_ensembleset(setup_tmpdir):
    mock_norne_data(reals=[0, 1], iters=[0, 1], parameters=True)


@pytest.fixture()
def norne_mocked_ensembleset_noparams(setup_tmpdir):
    mock_norne_data(reals=[0, 1], iters=[0, 1], parameters=False)


@pytest.fixture()
def setup_tmpdir(tmpdir):
    cwd = os.getcwd()
    tmpdir.chdir()
    yield
    os.chdir(cwd)



#!/usr/bin/env python
# pylint: disable=too-many-statements
from datetime import datetime

from ecl.summary import EclSum

from res.test.synthesizer import OilSimulator


def globalIndex(i, j, k, nx=10, ny=10, nz=10):
    return i + nx * (j - 1) + nx * ny * (k - 1)


def readParameters(filename):
    params = {}
    with open(filename, "r") as f:
        for line in f:
            key, value = line.split(":", 1)
            params[key] = value.strip()

    return params


def runSimulator(simulator, history_simulator, time_step_count):
    """ @rtype: EclSum """
    ecl_sum = EclSum.writer("SNAKE_OIL_FIELD", datetime(2010, 1, 1), 10, 10, 10)

    ecl_sum.addVariable("FOPT")
    ecl_sum.addVariable("FOPR")
    ecl_sum.addVariable("FGPT")
    ecl_sum.addVariable("FGPR")
    ecl_sum.addVariable("FWPT")
    ecl_sum.addVariable("FWPR")
    ecl_sum.addVariable("FGOR")
    ecl_sum.addVariable("FWCT")

    ecl_sum.addVariable("FOPTH")
    ecl_sum.addVariable("FOPRH")
    ecl_sum.addVariable("FGPTH")
    ecl_sum.addVariable("FGPRH")
    ecl_sum.addVariable("FWPTH")
    ecl_sum.addVariable("FWPRH")
    ecl_sum.addVariable("FGORH")
    ecl_sum.addVariable("FWCTH")

    ecl_sum.addVariable("WOPR", wgname="OP1")
    ecl_sum.addVariable("WOPR", wgname="OP2")
    ecl_sum.addVariable("WWPR", wgname="OP1")
    ecl_sum.addVariable("WWPR", wgname="OP2")
    ecl_sum.addVariable("WGPR", wgname="OP1")
    ecl_sum.addVariable("WGPR", wgname="OP2")
    ecl_sum.addVariable("WGOR", wgname="OP1")
    ecl_sum.addVariable("WGOR", wgname="OP2")
    ecl_sum.addVariable("WWCT", wgname="OP1")
    ecl_sum.addVariable("WWCT", wgname="OP2")

    ecl_sum.addVariable("WOPRH", wgname="OP1")
    ecl_sum.addVariable("WOPRH", wgname="OP2")
    ecl_sum.addVariable("WWPRH", wgname="OP1")
    ecl_sum.addVariable("WWPRH", wgname="OP2")
    ecl_sum.addVariable("WGPRH", wgname="OP1")
    ecl_sum.addVariable("WGPRH", wgname="OP2")
    ecl_sum.addVariable("WGORH", wgname="OP1")
    ecl_sum.addVariable("WGORH", wgname="OP2")
    ecl_sum.addVariable("WWCTH", wgname="OP1")
    ecl_sum.addVariable("WWCTH", wgname="OP2")

    ecl_sum.addVariable("BPR", num=globalIndex(5, 5, 5))
    ecl_sum.addVariable("BPR", num=globalIndex(1, 3, 8))

    time_map = []
    mini_step_count = 10
    total_step_count = time_step_count * mini_step_count

    for report_step in range(time_step_count):
        for mini_step in range(mini_step_count):
            t_step = ecl_sum.addTStep(
                report_step + 1, sim_days=report_step * mini_step_count + mini_step
            )

            time_map.append(t_step.getSimTime().datetime().strftime("%d/%m/%Y"))

            simulator.step(scale=1.0 / total_step_count)
            history_simulator.step(scale=1.0 / total_step_count)

            t_step["FOPR"] = simulator.fopr()
            t_step["FOPT"] = simulator.fopt()
            t_step["FGPR"] = simulator.fgpr()
            t_step["FGPT"] = simulator.fgpt()
            t_step["FWPR"] = simulator.fwpr()
            t_step["FWPT"] = simulator.fwpt()
            t_step["FGOR"] = simulator.fgor()
            t_step["FWCT"] = simulator.fwct()

            t_step["WOPR:OP1"] = simulator.opr("OP1")
            t_step["WOPR:OP2"] = simulator.opr("OP2")

            t_step["WGPR:OP1"] = simulator.gpr("OP1")
            t_step["WGPR:OP2"] = simulator.gpr("OP2")

            t_step["WWPR:OP1"] = simulator.wpr("OP1")
            t_step["WWPR:OP2"] = simulator.wpr("OP2")

            t_step["WGOR:OP1"] = simulator.gor("OP1")
            t_step["WGOR:OP2"] = simulator.gor("OP2")

            t_step["WWCT:OP1"] = simulator.wct("OP1")
            t_step["WWCT:OP2"] = simulator.wct("OP2")

            t_step["BPR:5,5,5"] = simulator.bpr("5,5,5")
            t_step["BPR:1,3,8"] = simulator.bpr("1,3,8")

            t_step["FOPRH"] = history_simulator.fopr()
            t_step["FOPTH"] = history_simulator.fopt()
            t_step["FGPRH"] = history_simulator.fgpr()
            t_step["FGPTH"] = history_simulator.fgpt()
            t_step["FWPRH"] = history_simulator.fwpr()
            t_step["FWPTH"] = history_simulator.fwpt()
            t_step["FGORH"] = history_simulator.fgor()
            t_step["FWCTH"] = history_simulator.fwct()

            t_step["WOPRH:OP1"] = history_simulator.opr("OP1")
            t_step["WOPRH:OP2"] = history_simulator.opr("OP2")

            t_step["WGPRH:OP1"] = history_simulator.gpr("OP1")
            t_step["WGPRH:OP2"] = history_simulator.gpr("OP2")

            t_step["WWPRH:OP1"] = history_simulator.wpr("OP1")
            t_step["WWPRH:OP2"] = history_simulator.wpr("OP2")

            t_step["WGORH:OP1"] = history_simulator.gor("OP1")
            t_step["WGORH:OP2"] = history_simulator.gor("OP2")

            t_step["WWCTH:OP1"] = history_simulator.wct("OP1")
            t_step["WWCTH:OP2"] = history_simulator.wct("OP2")

    return ecl_sum, time_map


def roundedInt(value):
    return int(round(float(value)))


if __name__ == "__main__":
    seed = int(readParameters("seed.txt")["SEED"])
    parameters = readParameters("snake_oil_params.txt")

    op1_divergence_scale = float(parameters["OP1_DIVERGENCE_SCALE"])
    op2_divergence_scale = float(parameters["OP2_DIVERGENCE_SCALE"])
    op1_persistence = float(parameters["OP1_PERSISTENCE"])
    op2_persistence = float(parameters["OP2_PERSISTENCE"])
    op1_offset = float(parameters["OP1_OFFSET"])
    op2_offset = float(parameters["OP2_OFFSET"])
    bpr_138_persistence = float(parameters["BPR_138_PERSISTENCE"])
    bpr_555_persistence = float(parameters["BPR_555_PERSISTENCE"])

    op1_octaves = roundedInt(parameters["OP1_OCTAVES"])
    op2_octaves = roundedInt(parameters["OP2_OCTAVES"])

    simulator = OilSimulator()
    simulator.addWell(
        "OP1",
        seed * 997,
        persistence=op1_persistence,
        octaves=op1_octaves,
        divergence_scale=op1_divergence_scale,
        offset=op1_offset,
    )
    simulator.addWell(
        "OP2",
        seed * 13,
        persistence=op2_persistence,
        octaves=op2_octaves,
        divergence_scale=op2_divergence_scale,
        offset=op2_offset,
    )
    simulator.addBlock("5,5,5", seed * 37, persistence=bpr_555_persistence)
    simulator.addBlock("1,3,8", seed * 31, persistence=bpr_138_persistence)

    history_simulator = OilSimulator()
    history_simulator.addWell("OP1", 222118781)
    history_simulator.addWell("OP2", 118116362)

    report_step_count = 200
    ecl_sum, time_map = runSimulator(simulator, history_simulator, report_step_count)

    ecl_sum.fwrite()

    with open("time_map.txt", "w") as f:
        for t in time_map:
            f.write("%s\n" % t)



from semeio.jobs.overburden_timeshift.ots import ots_run
from ecl.util.geometry import Surface
from ecl.grid import EclGrid
import pytest
from ots_util import mock_segy
import yaml
import numpy as np
import os


# pylint: disable=too-many-statements
@pytest.mark.parametrize(
    "res_scale, size_scale, pos_shift, results, surf_res",
    [
        # all scales or shifts are relative to the input grid file
        # segy file resolution, volume size, position of segy volume
        ((1, 1, 1), (1, 1), (0, 0), (True, True, False), (112, 46)),
        ((3, 3, 3), (2, 2), (0.5, 0.5), (True, False, True), (325, 138)),
        ((1, 1, 1), (2, 2), (0.5, 0.5), (True, False, False), (112, 46)),
        ((1, 1, 1), (2, 2), (-0.5, -0.5), (False, True, False), (112, 46)),
    ],
)
def test_ots_config_run_parameters(
    ots_tmpdir_enter, res_scale, size_scale, pos_shift, results, surf_res
):
    eclcase_dir = ots_tmpdir_enter
    grid_file = os.path.join(eclcase_dir, "NORNE_ATW2013.EGRID")
    grid = EclGrid(grid_file, apply_mapaxes=True)

    mock_segy(grid, res_scale, size_scale, pos_shift, "norne_vol.segy")

    conf = {
        "eclbase": os.path.join(eclcase_dir, "NORNE_ATW2013"),
        "above": 100,
        "seabed": 300,
        "youngs": 0.5,
        "poisson": 0.3,
        "rfactor": 20,
        "mapaxes": False,
        "convention": 1,
        "output_dir": "ts",
        "horizon": "horizon.irap",
        "vintages_export_file": "ts.txt",
        "velocity_model": "norne_vol.segy",
        "vintages": {
            "ts_simple": [["1997-11-06", "1998-02-01"], ["1997-12-17", "1998-01-01"]],
            "dpv": [["1997-11-06", "1997-12-17"]],
        },
    }
    with open("ots_config.yml", "w") as f:
        yaml.dump(conf, f, default_flow_style=False)
    ots_run("ots_config.yml")

    # test results
    # 1. we compare the surface resolution
    # 2. we test sum of output surface heightfields for
    #   2a. top left and bottom right corners (which is due to shift segy volume)
    #   2b. we expect the surface to drop with each timeshift
    def to_numpy(surf):
        arr = np.array([surf[i] for i in range(len(surf))]).reshape(-1, surf.getNX())
        return arr

    # horizon
    s_horizon = Surface("horizon.irap")
    assert surf_res[0] == s_horizon.getNX()
    assert surf_res[1] == s_horizon.getNY()
    assert (s_horizon.getNX() > grid.getNY()) == results[-1]
    assert (s_horizon.getNY() > grid.getNX()) == results[-1]

    _err = 0.01
    nx = s_horizon.getNX()
    ny = s_horizon.getNY()
    arr = to_numpy(s_horizon)
    sh_top_left = arr[: ny // 2 - 1, : nx // 2 - 1].sum()
    sh_bottom_right = arr[ny // 2 :, nx // 2 :].sum()
    assert ((sh_top_left - _err) > 0) == results[0]
    assert ((sh_bottom_right - _err) > 0) == results[1]

    # ts_simple1
    ts_simple1 = Surface("ts_ts_simple/ots_1997_11_06_1998_02_01.irap")
    assert nx == ts_simple1.getNX()
    assert ny == ts_simple1.getNY()
    assert (ts_simple1.getNX() > grid.getNY()) == results[-1]
    assert (ts_simple1.getNY() > grid.getNX()) == results[-1]

    arr = to_numpy(ts_simple1)
    ts1_top_left = arr[: ny // 2 - 1, : nx // 2 - 1].sum()
    ts1_bottom_right = arr[ny // 2 :, nx // 2 :].sum()
    assert ((ts1_top_left - _err) > 0) == results[0]
    assert ((ts1_bottom_right - _err) > 0) == results[1]
    assert ts1_top_left + ts1_bottom_right < sh_top_left + sh_bottom_right

    # ts_simple2
    ts_simple2 = Surface("ts_ts_simple/ots_1997_12_17_1998_01_01.irap")
    assert nx == ts_simple2.getNX()
    assert ny == ts_simple2.getNY()
    assert (ts_simple2.getNX() > grid.getNY()) == results[-1]
    assert (ts_simple2.getNY() > grid.getNX()) == results[-1]

    arr = to_numpy(ts_simple2)
    ts2_top_left = arr[: ny // 2 - 1, : nx // 2 - 1].sum()
    ts2_bottom_right = arr[ny // 2 :, nx // 2 :].sum()
    assert ((ts2_top_left - _err) > 0) == results[0]
    assert ((ts2_bottom_right - _err) > 0) == results[1]
    assert ts2_top_left + ts2_bottom_right < sh_top_left + sh_bottom_right
    assert ts2_top_left + ts2_bottom_right < ts1_top_left + ts1_bottom_right

    # ts_dpv
    ts_dpv = Surface("ts_dpv/ots_1997_11_06_1997_12_17.irap")
    assert nx == ts_dpv.getNX()
    assert ny == ts_dpv.getNY()
    assert (ts_dpv.getNX() > grid.getNY()) == results[-1]
    assert (ts_dpv.getNY() > grid.getNX()) == results[-1]

    arr = to_numpy(ts_simple2)
    dpv_top_left = arr[: ny // 2 - 1, : nx // 2 - 1].sum()
    dpv_bottom_right = arr[ny // 2 :, nx // 2 :].sum()
    assert ((dpv_top_left - _err) > 0) == results[0]
    assert ((dpv_bottom_right - _err) > 0) == results[1]
    assert dpv_top_left + dpv_bottom_right < sh_top_left + sh_bottom_right



"""Test the Trajectory and TrajectoryPoint classes with binary Eclipse data"""

import datetime

import numpy as np

from ecl.grid import EclGrid
from ecl.rft import EclRFTFile

from semeio.jobs.rft.trajectory import TrajectoryPoint, Trajectory
from tests.jobs.rft import conftest

ECL_BASE_REEK = conftest.get_ecl_base_reek()
ECL_BASE_NORNE = conftest.get_ecl_base_norne()


def test_update_simdata_from_rft_reek(reek_data):
    """Test data extraction from the binary Eclipse files
    for a single well point using TrajectoryPoint.update_simdata_from_rft()"""

    grid = EclGrid(ECL_BASE_REEK + ".EGRID")
    rft = EclRFTFile(ECL_BASE_REEK + ".RFT")
    rft_well_date = rft.get("OP_1", datetime.date(2000, 2, 1))

    # A trajectory point for an active cell in reek:
    point = TrajectoryPoint(462608.57, 5934210.96, 1624.38, 1624.38)
    assert point.grid_ijk is None
    assert point.pressure is None
    assert point.swat is None
    assert point.sgas is None
    assert point.soil is None
    point.set_ijk(grid.find_cell(point.utm_x, point.utm_y, point.true_vertical_depth))
    assert point.grid_ijk == (28, 27, 7)
    point.update_simdata_from_rft(rft_well_date)
    assert np.isclose(point.pressure, 304.37)
    assert np.isclose(point.swat, 0.151044)
    assert np.isclose(point.soil, 1 - 0.151044)
    assert np.isclose(point.sgas, 0.0)

    # Construct a Trajectory from the point
    traj = Trajectory([])
    traj.trajectory_points = [point]  # (can't initialize from list of points)
    dframe = traj.to_dataframe()
    assert {"i", "j", "k", "pressure", "soil", "sgas", "swat"}.issubset(set(dframe))


def test_update_simdata_from_rft_norne(norne_data):
    """Similar test as the reek version, but the Norne RFT file
    does not contain saturations, and in libecl terms contains EclPLTCell
    as opposed to EclRFTCell as in Reek"""

    grid = EclGrid(ECL_BASE_NORNE + ".EGRID")
    rft = EclRFTFile(ECL_BASE_NORNE + ".RFT")
    rft_well_date = rft.get("C-3H", datetime.date(1999, 5, 4))

    # A trajectory point for an active cell in Norne, picked
    # from a line in gendata_rft_input_files/C-3H.txt
    point = TrajectoryPoint(
        455752.59771598293, 7321015.949386452, 2785.78173828125, 2785.78173828125
    )
    assert point.grid_ijk is None
    assert point.pressure is None
    assert point.swat is None
    assert point.sgas is None
    assert point.soil is None
    point.set_ijk(grid.find_cell(point.utm_x, point.utm_y, point.true_vertical_depth))
    assert point.grid_ijk == (8, 12, 20)  # Zero-indexed integers.
    point.update_simdata_from_rft(rft_well_date)
    # There is no saturation data in the Norne binary output, then these
    # should be None
    assert point.swat is None
    assert point.sgas is None
    assert point.soil is None

    # Construct a Trajectory from the point
    traj = Trajectory([])
    traj.trajectory_points = [point]  # (can't initialize from list of points)
    dframe = traj.to_dataframe()
    assert {"i", "j", "k", "pressure"}.issubset(set(dframe))
    assert "swat" not in dframe


def test_update_simdata_outside_grid(tmpdir):
    grid = EclGrid(ECL_BASE_REEK + ".EGRID")
    rft = EclRFTFile(ECL_BASE_REEK + ".RFT")
    rft_well_date = rft.get("OP_1", datetime.date(2000, 2, 1))

    # A point outside the grid:
    point = TrajectoryPoint(45000, 60000000, 1, 1)
    point.set_ijk(grid.find_cell(point.utm_x, point.utm_y, point.true_vertical_depth))
    assert point.grid_ijk is None  # There is no Exception raised by set_ijk()

    point.update_simdata_from_rft(rft_well_date)
    assert point.pressure is None  # Since we are outside the grid.

    # Construct a Trajectory from the point
    traj = Trajectory([])
    traj.trajectory_points = [point]  # (can't initialize from list of points)
    dframe = traj.to_dataframe()
    assert not set(dframe).intersection({"i", "j", "k", "pressure", "swat"})


def test_update_simdata_outside_well(tmpdir):
    grid = EclGrid(ECL_BASE_REEK + ".EGRID")
    rft = EclRFTFile(ECL_BASE_REEK + ".RFT")
    rft_well_date = rft.get("OP_1", datetime.date(2000, 2, 1))

    # A point in the grid, but not related to the well
    point = TrajectoryPoint(462825.55, 5934025.52, 1623.19, 1623.19)
    point.set_ijk(grid.find_cell(point.utm_x, point.utm_y, point.true_vertical_depth))
    # NB: grid_ijk ints start at zero, ResInsight and ecl2df report this as (29, 29, 7)
    assert point.grid_ijk == (28, 28, 6)
    point.update_simdata_from_rft(rft_well_date)
    assert point.pressure is None
    assert point.swat is None
    assert point.sgas is None
    assert point.soil is None

    # Construct a Trajectory from the point
    traj = Trajectory([])
    traj.trajectory_points = [point]  # (can't initialize from list of points)
    dframe = traj.to_dataframe()
    assert {"i", "j", "k"}.issubset(set(dframe))
    assert not set(dframe).intersection({"pressure", "swat", "soil", "sgas"})



import sys
import subprocess
import pytest

from ert_shared.plugins.plugin_manager import ErtPluginContext
import semeio.hook_implementations.jobs
import ert_shared.hook_implementations


default_config = """
JOBNAME TEST

QUEUE_SYSTEM LOCAL
QUEUE_OPTION LOCAL MAX_RUNNING 1


NUM_REALIZATIONS 1
MIN_REALIZATIONS 1

FORWARD_MODEL {}({})
"""


@pytest.mark.skipif(
    sys.platform == "darwin",
    reason=(
        "Forward models have same name as executable and "
        "libres will find wrong exe on case insensitive system "
        "bug report here: https://github.com/equinor/libres/issues/1053"
    ),
)
@pytest.mark.parametrize(
    "forward_model, configuration, expected_error",
    [
        ("OTS", "<CONFIG>=config.ots", "config.ots is not an existing file!"),
        ("DESIGN2PARAMS", "<IENS>=not_int", "invalid int value: 'not_int'"),
        (
            "DESIGN_KW",
            "<template_file>=no_template",
            " no_template is not an existing file!",
        ),
        ("GENDATA_RFT", "<ECL_BASE>=not_ecl", "The path not_ecl.RFT does not exist"),
        ("PYSCAL", "<PARAMETER_FILE>=not_file", "not_file does not exist"),
        ("STEA", "<CONFIG>=not_a_file", "not_a_file is not an existing file!"),
    ],
)
def test_console_script_integration(
    setup_tmpdir, forward_model, configuration, expected_error
):
    config = default_config.format(forward_model, configuration)
    with open("config.ert", "w") as fh:
        fh.write(config)

    with ErtPluginContext(
        plugins=[semeio.hook_implementations.jobs, ert_shared.hook_implementations]
    ):
        subprocess.run(["ert", "test_run", "config.ert", "--verbose"], check=False)
    with open(f"simulations/realization0/{forward_model}.stderr.0") as fin:
        error = fin.read()
    assert expected_error in error



from semeio.workflows.correlated_observations_scaling import ObservationScaleFactor
from semeio.workflows.misfit_preprocessor.config import (
    AUTO_SCALE,
    SPEARMAN_CORRELATION,
)
from semeio.workflows.spearman_correlation_job.job import spearman_job


def run(config, measured_data, reporter):

    if config.clustering.method == SPEARMAN_CORRELATION:
        sconfig = config.clustering.spearman_correlation
        scaling_configs = spearman_job(
            measured_data,
            sconfig.fcluster.t,
            reporter,
            criterion=sconfig.fcluster.criterion,
            depth=sconfig.fcluster.depth,
            method=sconfig.linkage.method,
            metric=sconfig.linkage.metric,
        )
    elif config.clustering.method == AUTO_SCALE:
        job = ObservationScaleFactor(reporter, measured_data)
        nr_components, _ = job.perform_pca(config.scaling.threshold)
        sconfig = config.clustering.auto_scale
        scaling_configs = spearman_job(
            measured_data,
            nr_components,
            reporter,
            criterion="maxclust",
            method=sconfig.linkage.method,
            metric=sconfig.linkage.metric,
        )
    else:
        raise AssertionError(
            "Unknown clustering method: {}".format(config.clustering.method)
        )

    return scaling_configs



import numpy as np
import pandas as pd

import warnings
import logging

from semeio._exceptions.exceptions import ValidationError

# Filenames created/modified by this script.
# Don't change unless you understand the consequences.
_TARGET_FILE_TXT = "DESIGN2PARAMS.OK"
_DESIGN_MATRIX_TXT = "designmatrix.txt"
_DESIGN_PARAMETERS_TXT = "designparameters.txt"
_PARAMETERS_TXT = "parameters.txt"

# These parameter names are reserved and cannot
# be used in design matrices.
DENYLIST = ["ENSEMBLE", "DATE"]
DENYLIST_DEFAULTS = DENYLIST + ["REAL"]

logger = logging.getLogger(__name__)


def run(
    realization,
    xlsfilename,
    designsheetname="DesignSheet01",
    defaultssheetname="DefaultValues",
    parametersfilename="parameters.txt",
    log_level=None,
):
    """
    Reads out all file content from different files and create dataframes
    """
    if log_level is not None:
        logger.setLevel(log_level)

    design_matrix_sheet = _read_excel(xlsfilename, designsheetname)

    try:
        _validate_design_matrix_header(design_matrix_sheet)
    except ValueError as err:
        raise ValidationError(f"Design matrix not valid, error: {str(err)}") from err

    if realization in _invalid_design_realizations(design_matrix_sheet):
        raise SystemExit("Design parameters invalid for current realization")

    default_df = _read_defaultssheet(xlsfilename, defaultssheetname)

    try:
        parameters = pd.read_csv(parametersfilename, delimiter=" ", header=None)
    except IOError:
        logger.info(
            "No {} exists, creating a new, empty one.".format(parametersfilename)
        )
        parameters = pd.DataFrame(columns=[0, 1])
    except pd.errors.EmptyDataError:
        logger.info("{} existed but was empty.".format(parametersfilename))
        parameters = pd.DataFrame(columns=[0, 1])
    parameters.rename(columns={0: "keys", 1: "parameters"}, inplace=True)

    _complete_parameters_file(
        realization, parameters, parametersfilename, design_matrix_sheet, default_df
    )


def _complete_parameters_file(
    realization, parameters, parametersfilename, design_matrix_sheet, default_sheet
):
    """
    Pick key / values from choosen realization in design matrix
    Append those key / values if not present into parameters.txt
    Create file adding those key / values to designparameters.txt
    if default-sheet contains key / values not existing in
    design matrix nor parameters, append those as well to both
    parameters.txt and designparameters.txt
    Write designmatrix.txt as a csv repr of xls

    :raises: SystemExit if design matrix contains empty headers
    :raises: SystemExit design matrix contains empty cells
    :raises: SystemExit if provided realization not exists
    """
    # get header / vals for chosen realization
    try:
        realization_values = pd.DataFrame(design_matrix_sheet.iloc[realization, 1:])
    except IndexError as err:
        raise SystemExit(
            "Provided realization arg {} does not exist in design matrix".format(
                realization
            )
        ) from err
    realization_values.reset_index(inplace=True)
    realization_values.rename(
        columns={"index": "keys", realization: "realization"}, inplace=True
    )

    # merge realization from design-matrix with parameters and defaults to one dataframe
    merged = pd.merge(
        parameters,
        pd.merge(realization_values, default_sheet, on="keys", how="outer"),
        on="keys",
        how="outer",
    )

    # add new column with a combined parameters / data
    # from the realization in design matrix
    merged["parameters_realization"] = merged["parameters"].combine_first(
        merged["realization"]
    )

    # add new column with a total combined of parameters_realization and defaults
    merged["combined"] = merged["parameters_realization"].combine_first(
        merged["defaults"]
    )

    # keys in the design matrix not present in parameters
    design_parameters = merged[
        merged["parameters"].isnull() & merged["realization"].notnull()
    ]
    logger.info(
        "\ndesign parameters used: \n%s", design_parameters[["keys", "combined"]]
    )

    # keys without value in parameters and design matrix
    defaults = merged[merged["parameters_realization"].isnull()]
    logger.info("\ndefaults used: \n%s", defaults[["keys", "combined"]])

    # warn if keys with different values in parameters.tst and design matrix
    conflicts = merged[
        ~(merged["combined"].astype(str) == merged["realization"].astype(str))
        & (~merged["realization"].isnull())
    ]
    if not conflicts.empty:
        for _, row in conflicts.iterrows():
            msg = (
                "Parameter {} already exists in {} with value {}, "
                "design matrix value {} ignored"
            ).format(
                row["keys"], parametersfilename, row["parameters"], row["realization"]
            )
            logger.warning(msg)

    # write used design matrix to csv
    design_matrix_sheet.to_csv(
        _DESIGN_MATRIX_TXT,
        sep=" ",
        index=False,
        float_format="%.10f",
        na_rep="__VOID__",
    )

    # keys not present in parameters
    combined_parameters = merged[merged["parameters"].isnull()]

    # append new keys to parameters.txt
    combined_parameters.to_csv(
        parametersfilename,
        columns=["keys", "combined"],
        sep=" ",
        mode="a",
        header=False,
        index=False,
    )

    # write file with the new keys to designparameters.txt
    combined_parameters.to_csv(
        _DESIGN_PARAMETERS_TXT,
        columns=["keys", "combined"],
        sep=" ",
        header=False,
        index=False,
    )

    # if all ok - write the ok file
    with open(_TARGET_FILE_TXT, "w") as target_file:
        target_file.write("OK\n")


def _read_excel(file_name, sheet_name, header=0):
    """
    Make dataframe from excel file
    :return: Dataframe
    :raises: SystemExit if file not found
    :raises: SystemExit if file not loaded correctly
    """
    try:
        return pd.read_excel(file_name, sheet_name, header=header, dtype=str)
    except IOError as err:
        raise SystemExit("File {} not found".format(file_name)) from err
    except Exception as err:
        raise SystemExit(
            (
                "File {} is probably not of correct type. Failed with exception '{}'"
            ).format(file_name, str(err))
        ) from err


def _validate_design_matrix_header(design_matrix):
    """
    Validate header in user inputted design matrix
    :raises: ValueError if design matrix contains empty headers
    """
    try:
        unnamed = design_matrix.loc[:, design_matrix.columns.str.contains("^Unnamed")]
    except ValueError as err:
        # We catch because int/floats as column headers
        # in xlsx gets read as int/float and is not valid to index by.
        raise ValueError(
            f"Invalid value in design matrix header, error: {str(err)}"
        ) from err
    column_indexes = [int(x.split(":")[1]) for x in unnamed.columns.values]
    if len(column_indexes) > 0:
        raise ValueError(
            "Column headers not present in column {}".format(column_indexes)
        )


def _invalid_design_realizations(design_matrix):
    """
    Build a set of realization indices where something is wrong,
    f.ex empty cells

    Logs warnings for all empty cells found in all realizations.

    Logs warning if it looks like some column names have been
    repeated in the source xlsx file.

    :raises: SystemExit if some parameter names are not allowed
    """

    empty_cell_coords = list(zip(*np.where(pd.isnull(design_matrix))))

    empties = [
        "Realization {}, column {}".format(i, design_matrix.columns[j])
        for i, j in zip(*np.where(pd.isnull(design_matrix)))
    ]
    if len(empties) > 0:
        logger.warning("Design matrix contains empty cells {}".format(empties))

    # Look for initial or trailing whitespace in column headers. This
    # is disallowed as it can create user confusion and has no use-case.
    for col_header in design_matrix:
        if col_header != col_header.strip():
            raise SystemExit(
                'Column header "{}" contains initial or trailing whitespace.'.format(
                    col_header
                )
            )

    # pd.read_excel() will always allow duplicated column names, but will
    # change names of the columns so that if REAL is given twice, it will
    # occur as "REAL" and "REAL.1". Give warning if it looks like that
    # has happened.
    dot1columns = [
        col
        for col in design_matrix.columns
        if col.endswith(".1") and col.replace(".1", "") in design_matrix
    ]
    if dot1columns:
        logger.warning(
            "Column(s) {} are probably duplicated in design matrix".format(dot1columns)
        )

    denied_params = set(design_matrix.columns).intersection(set(DENYLIST))
    if denied_params:
        raise SystemExit(
            "Column name(s) {} is not allowed in design matrix.".format(denied_params)
        )

    return {cell_coord[0] for cell_coord in empty_cell_coords}


def _read_defaultssheet(xlsfilename, defaultssheetname):
    """
    Reads a XLSX file and tries to return a dataframe
    with defaults for design matrix parameters.

    A dataframe is always returned, possibly empty.
    Columns are always exactly named "keys" and "defaults"

    :raises: SystemExit if defaults sheet is non-empty but non-parsable
    """
    if defaultssheetname:
        default_df = _read_excel(xlsfilename, defaultssheetname, header=None)
        if default_df.empty:
            logger.info("Empty defaultssheet provided")
            default_df = pd.DataFrame(columns=[0, 1])
        if len(default_df.columns) < 2:
            raise SystemExit(
                "Defaults sheet must have exactly two columns, only one found"
            )
        if len(default_df.columns) > 2:
            warnings.warn(
                (
                    "DEPRECATION: You supplied more than two columns for "
                    "the default values. This is not supported and will stop "
                    "working in the future."
                ),
                UserWarning,
            )
            default_df = default_df[[0, 1]]  # Slicing columns
        # Look for initial or trailing whitespace in parameter names. This
        # is disallowed as it can create user confusion and has no use-case.
        for paramname in default_df.loc[:, 0]:
            if paramname != paramname.strip():
                raise SystemExit(
                    (
                        'Parameter name "{}" in default values contains '
                        "initial or trailing whitespace."
                    ).format(paramname)
                )

        denied_params = set(default_df.loc[:, 0]).intersection(set(DENYLIST_DEFAULTS))
        if denied_params:
            raise SystemExit(
                "Column name(s) {} is not allowed in design matrix defaults.".format(
                    denied_params
                )
            )

    else:
        logger.info("No defaultssheet provided, using empty dataframe")
        default_df = pd.DataFrame(columns=[0, 1])

    default_df.rename(columns={0: "keys", 1: "defaults"}, inplace=True)
    return default_df



import os
import copy
from distutils.dir_util import copy_tree

import numpy
import pytest

ECL_BASE_NORNE = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "data/norne/NORNE_ATW2013"
)
ECL_BASE_REEK = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "data/reek/2_R001_REEK-0"
)
EXPECTED_RESULTS_PATH_NORNE = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "data/norne/expected_result"
)

MOCK_DATA_CONTENT_NORNE = {
    "RFT_B-1AH_0_active": [1] * 5,
    "RFT_B-4AH_0_active": [1] * 7 + [0] * 1 + [1] * 21,
    "RFT_B-4H_0_active": [1] * 20,
    "RFT_C-1H_0_active": [1] * 21,
    "RFT_C-3H_0_active": [1] * 21,
    "RFT_C-4AH_0_active": [1] * 15,
}


def get_ecl_base_norne():
    return copy.copy(ECL_BASE_NORNE)


def get_ecl_base_reek():
    return copy.copy(ECL_BASE_REEK)


def get_expected_results_path_norne():
    return copy.copy(EXPECTED_RESULTS_PATH_NORNE)


def get_mock_data_content_norne():
    return copy.deepcopy(MOCK_DATA_CONTENT_NORNE)


def _assert_almost_equal_line_by_line(file1, file2):
    with open(file1) as fh:
        file1_content = fh.readlines()

    with open(file2) as fh:
        file2_content = fh.readlines()

    assert len(file1_content) == len(file2_content)

    for line1, line2 in zip(file1_content, file2_content):
        try:
            line1, line2 = float(line1), float(line2)
        except ValueError:
            continue
        numpy.testing.assert_almost_equal(line1, line2, decimal=7)


def _generate_mock_data_norne(write_directory):
    for fname, content in MOCK_DATA_CONTENT_NORNE.items():
        with open(os.path.join(write_directory, fname), "w+") as fh:
            fh.write("\n".join([str(c) for c in content]))


@pytest.fixture
def norne_data(tmpdir):
    data_dir = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "data/norne/gendata_rft_input_files"
    )
    copy_tree(data_dir, tmpdir.strpath)

    cwd = os.getcwd()
    tmpdir.chdir()

    expected_results_dir = os.path.join(tmpdir.strpath, "expected_results")
    os.mkdir(expected_results_dir)

    copy_tree(EXPECTED_RESULTS_PATH_NORNE, expected_results_dir)
    _generate_mock_data_norne(expected_results_dir)

    try:
        yield

    finally:
        os.chdir(cwd)


@pytest.fixture
def reek_data(tmpdir):
    data_dir = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "data/reek/gendata_rft_input_files"
    )
    copy_tree(data_dir, tmpdir.strpath)
    cwd = os.getcwd()
    tmpdir.chdir()

    try:
        yield

    finally:
        os.chdir(cwd)



import shutil
import yaml
import pytest
from semeio._exceptions.exceptions import ConfigurationError
from semeio.jobs.overburden_timeshift.ots import ots_load_params
from pathlib import Path


TEST_NORNE_DIR = Path(__file__).parent / ".." / ".." / "test_data" / "norne"


@pytest.fixture()
def ecl_files(tmpdir):
    for extension in ["INIT", "EGRID", "UNRST"]:
        shutil.copy(TEST_NORNE_DIR / f"NORNE_ATW2013.{extension}", tmpdir)


@pytest.mark.parametrize("vintages_export_file", ["ts.txt", None])
@pytest.mark.parametrize("horizon", ["horizon.irap", None])
@pytest.mark.parametrize("velocity_model", ["norne_vol.segy", None])
def test_valid_config(
    tmpdir, monkeypatch, velocity_model, horizon, vintages_export_file, ecl_files
):
    conf = {
        "eclbase": "NORNE_ATW2013",
        "above": 100,
        "seabed": 300,
        "youngs": 0.5,
        "poisson": 0.3,
        "rfactor": 20,
        "mapaxes": False,
        "convention": 1,
        "output_dir": "ts",
        "vintages": {
            "ts_simple": [["1997-11-06", "1998-02-01"], ["1997-12-17", "1998-02-01"]],
            "dpv": [["1997-11-06", "1997-12-17"]],
        },
    }
    if velocity_model:
        conf.update({"velocity_model": velocity_model})
    if horizon:
        conf.update({"horizon": horizon})
    if vintages_export_file:
        conf.update({"vintages_export_file": vintages_export_file})
    with tmpdir.as_cwd():
        with open("ots_config.yml", "w") as f:
            yaml.dump(conf, f, default_flow_style=False)
        ots_load_params("ots_config.yml")


@pytest.mark.parametrize(
    "extension, error_msg",
    [
        ["INIT", "Eclbase must have an INIT file"],
        ["EGRID", "Eclbase must have an EGRID file"],
        ["UNRST", "Eclbase must have a UNRST file"],
    ],
)
def test_eclbase_config(tmpdir, monkeypatch, ecl_files, extension, error_msg):
    conf = {
        "eclbase": "NORNE_ATW2013",
        "above": 100,
        "seabed": 300,
        "youngs": 0.5,
        "poisson": 0.3,
        "rfactor": 20,
        "mapaxes": False,
        "convention": 1,
        "output_dir": "ts",
        "horizon": "horizon.irap",
        "vintages_export_file": "ts.txt",
        "velocity_model": "norne_vol.segy",
        "vintages": {
            "ts_simple": [["1997-11-06", "1998-02-01"], ["1997-12-17", "1998-02-01"]],
            "dpv": [["1997-11-06", "1997-12-17"]],
        },
    }
    with tmpdir.as_cwd():
        with open("ots_config.yml", "w") as f:
            yaml.dump(conf, f, default_flow_style=False)
        # Renaming a needed ecl file to simulate it not existing
        Path(f"NORNE_ATW2013.{extension}").rename("NOT_ECLBASE")
        with pytest.raises(ConfigurationError, match=error_msg):
            ots_load_params("ots_config.yml")



import argparse
import sys

import pandas as pd
from ert_shared.plugins.plugin_manager import hook_implementation
from fmu import ensemble
from res.job_queue import ErtScript

DESCRIPTION = """
CSV_EXPORT2 will export selected Eclipse summary vectors to a CSV file.
The vector selection is independent of the ``SUMMARY`` keywords in the
ert config file.

The CSV file will look like:

======== ==== =========== ==== ======
ENSEMBLE REAL DATE        FOPR FOPT
======== ==== =========== ==== ======
iter-0   0    2020-01-01  800  0
iter-0   0    2020-02-01  1000 365000
iter-0   1    2020-01-01  700  0
iter-0   1    2020-01-01  1100 401500
======== ==== =========== ==== ======

The time frequency must be chosen. If ``raw``, the original timesteps from
Eclipse is chosen, and it will be individual pr. realization. If ``daily``,
``weekly``, ``monthly``  or  ``yearly`` is chosen, only data at those dates are
given for all realization. Rate data (e.g.  FOPR) is valid for the given dates,
but can not be summed up to cumulative data when time interpolation. Cumulative
columns (f.ex. FOPT) are time-interpolated linearly. See the `documentation on
fmu-ensemble
<https://equinor.github.io/fmu-ensemble/usage.html#rate-handling-in-eclipse-summary-vectors>`_
for more details on rate handling.

Columns are selected by a list of strings, where wildcards characters ``?``
(matches exactly one character) and ``*`` (matches zero or more characters) can
be used to select multiple columns.

Column count more than 1000 gives increased probability for problems downstream,
depending on which applications are put into use. Column count depends on the
combination of wildcards used in this workflow and the actual vectors that are
requested in the Eclipse DATA file. A wildcard like ``W*`` can in certain cases
(e.g. Eclipse simulations with 100+ wells) produce thousands of vectors, and can
then be replaced by something more explicit like ``WOPT* WGPT* WWPT*``.
"""  # noqa

EXAMPLES = """
Example
-------

Add a file named e.g. ``ert/bin/workflows/QC_CSVEXPORT2`` with the contents::

  MAKE_DIRECTORY <CASEDIR>/share/summary/
  EXPORT_RUNPATH * | *
  CSV_EXPORT2 <RUNPATH_FILE> <CASEDIR>/share/summary/<CASE>.csv monthly F* W* TCPU TIMESTEP

(where ``<CASEDIR>`` typically points to ``/scratch/..``). Adjust all three
lines to your needs.

``EXPORT_RUNPATH`` in the workflow file is added to ensure all realizations and
all iterations are included in the RUNPATH file.  If you have rerun only a
subset of your ensemble, the RUNPATH file will only contain those unless this
statement is included.

Add to your ERT config to have the workflow automatically executed on successful
runs::

  LOAD_WORKFLOW ../bin/workflows/QC_CSVEXPORT2
  HOOK_WORKFLOW QC_CSVEXPORT2 POST_SIMULATION

"""  # noqa


def csv_exporter(runpathfile, time_index, outputfile, column_keys=None):
    """Export CSV data (summary and parameters) from an EnsembleSet

    The EnsembleSet is described by a runpathfile which must exists
    and point to realizations"""
    ensemble_set = ensemble.EnsembleSet(
        name="ERT EnsembleSet for CSV_EXPORT2", runpathfile=runpathfile
    )
    summary = ensemble_set.load_smry(time_index=time_index, column_keys=column_keys)
    try:  # try/except is needed for fmu-ensemble<=1.3.0
        parameters = ensemble_set.parameters
    except KeyError:
        parameters = pd.DataFrame()

    if not parameters.empty:
        pd.merge(summary, parameters).to_csv(outputfile, index=False)
    else:
        summary.to_csv(outputfile, index=False)


class CsvExport2Job(ErtScript):
    def run(self, *args):
        main(args)


def main(args):
    parser = csv_export_parser()
    args = parser.parse_args(args)

    csv_exporter(
        runpathfile=args.runpathfile,
        time_index=args.time_index,
        outputfile=args.outputfile,
        column_keys=args.column_keys,
    )

    print("{} csv-export written to {}".format(args.time_index, args.outputfile))


def csv_export_parser():
    """Setup parser"""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "runpathfile",
        type=str,
        help=(
            "Path to ERT RUNPATH-file, "
            "usually the ERT magic variable <RUNPATH> can be used"
        ),
    )
    parser.add_argument(
        "outputfile",
        type=str,
        help="Path to CSV file to be written. The directory pointed to must exist.",
    )
    parser.add_argument(
        "time_index",
        type=str,
        default="monthly",
        help=(
            "Time interval specifier for the output. "
            "This argument is passed on to fmu-ensemble, "
            "supported specifiers are 'raw', 'daily', 'weekly', 'monthly' and 'yearly'"
        ),
    )
    parser.add_argument(
        "column_keys", nargs="+", default=None, help="List of summary vector wildcards"
    )
    return parser


@hook_implementation
def legacy_ertscript_workflow(config):
    workflow = config.add_workflow(CsvExport2Job, "CSV_EXPORT2")
    workflow.parser = csv_export_parser
    workflow.description = DESCRIPTION
    workflow.examples = EXAMPLES
    workflow.category = "export"


def cli():
    main(sys.argv[1:])



import os
import sys
import argparse
import logging

from semeio.jobs.rft.utility import (
    load_and_parse_well_time_file,
    valid_eclbase,
    existing_directory,
)
from semeio.jobs.rft import gendata_rft
from semeio.jobs.rft.trajectory import Trajectory
from semeio.jobs.rft.zonemap import ZoneMap

logger = logging.getLogger(__name__)

description = """
The gendata_rft application is used to extract RFT data (pressure and
saturation) for specific points in the reservoir (typically along well
trajectories) from an Eclipse output (or equivalent data format), given a file
containing relevant well names and their corresponding dates. It is also
possible to specify a zonemap that validates each zone specified in the
trajectory files.

Pressure data is ouputted to files like ``RFT_A-1_0`` in the selected
directory, this would correspond to well A-1 at report step 0. Saturation data
if available would be exported to ``RFT_A-1_SWAT_0``, and similarly for SGAS
and SOIL.

The user will be error messages and the applications stops if there is no RFT
available for the well(s) at the date(s).

A CSV file is prepared with all extracted data. This can be further used for
visualization in Webviz. In order to merge this CSV with observed values, see
merge_rft_ertobs from subscript.

The pressure or saturation data is set to -1 and the inactive value to 0 if any
of the following applies:

* The trajectory point can not be mapped to a cell in the grid
* There is no RFT data for the cell
* The zone mapping is invalid.

"""

examples = """
Setup a file with well-names and associated date of RFT data in a file called
e.g. ``well_date_rft.txt``::

    -- well DD MM YYYY report_step
    A-1     01 02 2000 0

A directory with trajectory files must be prepared, which must contain one
file for each well mentioned in the file above. A file in this directory
could look like::

    -- utmx    utmy      depth_MD depth_TVD  zone
    462608.57 5934210.96 1674.44  1624.38    Upper -- cell 29 28 2

(add more lines for more points).

A zonemap-file is a text-file with k-index and zone-name pr line, e.g a file
named ``layer_zone_table.txt``::

    1 Upper
    2 Upper
    3 Lower
    4 Lower

In the ert config, after running the Eclipse (or similiar) forward model, add::

    DEFINE RFT_INPUT <CONFIG_PATH>/../input/observations/rft
    FORWARD_MODEL GENDATA_RFT(<PATH_TO_TRAJECTORY_FILES>=<RFT_INPUT>/rft/, <WELL_AND_TIME_FILE>=<RFT_INPUT>/well_date_rft.txt, <ZONEMAP>=<RFT_INPUT>/layer_zone_table.txt)

For assisted history matching, add ``GEN_DATA`` statements to the ert config::

    GEN_DATA A-1 RESULT_FILE:RFT_A-1_%d INPUT_FORMAT:ASCII REPORT_STEPS:0

"""  # noqa
category = "utility.transformation"


def _build_parser():
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument(
        "-e",
        "--eclbase",
        type=valid_eclbase,
        required=True,
        help="""
Filepath to an eclipse base (or with equivalent data format) that
contains both RFT and EGRID files. Note that the proper extensions are added,
so it is only necessary to specify the basename
""",
    )
    parser.add_argument(
        "-w",
        "--well_and_time_file",
        type=load_and_parse_well_time_file,
        required=True,
        help="""
Filepath to a file containing the well and time that will be loaded from the RFT
file. The accepted file format is: <well_name> <DD> <MM> <YY> <report>.
For each well name given in the file there must also exist a <well_name.txt> file
that contains a list of trajectory points. Each line must be specified according
to: <utmx> <utmy> <TVD> <MD>
""",
    )
    parser.add_argument(
        "-t",
        "--trajectory_path",
        required=True,
        type=existing_directory,
        help="""
Path to the directory that contains the trajectory files referenced from the well
and time file. Each well must have a trajectory file that contains at least one
trajectory point.
""",
    )
    parser.add_argument(
        "-z",
        "--zonemap",
        required=False,
        type=ZoneMap.load_and_parse_zonemap_file,
        help="""
Path to a file with zonemap information. This file must include every k-value that
any RFT value is located within, irregardless if the trajectory point contains a
zone definition. If a k value does not exist the program will stop. The zone
mapping will be validated and if it is invalid, the pressure is set to -1 and the
inactive value to 0
""",
    )
    parser.add_argument(
        "-c",
        "--csvfile",
        required=False,
        type=str,
        default="gendata_rft.csv",
        help="""
Path to a file to which a CSV output is dumped. The CSV will contain all the
information of the wellspaths, times, pressures and zone information, and whether
a specific pressure point is valid
""",
    )
    parser.add_argument(
        "-o",
        "--outputdirectory",
        required=False,
        type=existing_directory,
        default=".",
        help=(
            "Output directory used for outputting resulting files "
            "(except for the csvfile output). "
            "The directory must exist. "
            "Defaults to RUNPATH."
        ),
    )
    parser.add_argument(
        "--log-level",
        "-l",
        required=False,
        default="WARNING",
        type=logging.getLevelName,
        help="Sets the log level",
    )

    return parser


def main_entry_point():
    arg_parser = _build_parser()
    options = arg_parser.parse_args()
    logger.setLevel(options.log_level)

    context_errors = []
    trajectories = {}
    for well_name, *_ in options.well_and_time_file:
        try:
            trajectories[well_name] = Trajectory.load_from_file(
                filepath=os.path.join(options.trajectory_path, well_name + ".txt")
            )
        except (IOError, ValueError) as err:
            context_errors.append(str(err))

    if context_errors:
        raise SystemExit("\n".join(context_errors))

    logger.info("All files loaded\nRetrieving RFT data...")

    try:
        gendata_rft.run(
            well_times=options.well_and_time_file,
            trajectories=trajectories,
            ecl_grid=options.eclbase[0],
            ecl_rft=options.eclbase[1],
            zonemap=options.zonemap,
            csvfile=options.csvfile,
            outputdirectory=options.outputdirectory,
        )
        with open("GENDATA_RFT.OK", "w") as fh:
            fh.write("GENDATA RFT completed OK")
        logger.info("Completed!")
    except ValueError as exception:
        logger.error(str(exception))
        sys.exit(1)



import pytest

from semeio.workflows.misfit_preprocessor.exceptions import ValidationError
from semeio.workflows.misfit_preprocessor import assemble_config


@pytest.mark.parametrize(
    "input_observations, expected_result",
    [
        (["o*"], ["observations", "of"]),
        (["of"], ["of"]),
        (["a", "list"], ["a", "list"]),
    ],
)
def test_assemble_config(input_observations, expected_result):
    config = assemble_config(
        {"observations": input_observations},
        ["a", "list", "of", "existing", "observations"],
    ).snapshot

    assert expected_result == sorted(config.observations)


def test_assemble_config_default_observations():
    config = assemble_config(
        {},
        ["a", "list", "of", "existing", "observations"],
    ).snapshot

    assert sorted(["a", "list", "of", "existing", "observations"]) == sorted(
        config.observations
    )


def test_assemble_config_not_existing_obs():
    with pytest.raises(ValidationError) as ve:
        assemble_config(
            {"observations": ["not_an_observation"]},
            ["a", "list", "of", "existing", "observations"],
        )

    expected_err_msg = (
        "Invalid configuration of misfit preprocessor\n"
        "  - Found no match for observation not_an_observation (observations.0)\n"
    )
    assert expected_err_msg == str(ve.value)



#!/usr/bin/env python
from ecl.summary import EclSum


def writeDiff(filename, ecl_sum, key1, key2):
    with open(filename, "w") as f:
        for v1, v2 in zip(ecl_sum.numpy_vector(key1), ecl_sum.numpy_vector(key2)):
            diff = v1 - v2
            f.write("%f\n" % diff)


if __name__ == "__main__":
    ecl_sum = EclSum("SNAKE_OIL_FIELD")

    report_step = 199
    writeDiff(
        "snake_oil_opr_diff_%d.txt" % report_step, ecl_sum, "WOPR:OP1", "WOPR:OP2"
    )
    writeDiff(
        "snake_oil_wpr_diff_%d.txt" % report_step, ecl_sum, "WWPR:OP1", "WWPR:OP2"
    )
    writeDiff(
        "snake_oil_gpr_diff_%d.txt" % report_step, ecl_sum, "WGPR:OP1", "WGPR:OP2"
    )



import pytest


@pytest.mark.script_launch_mode("subprocess")
@pytest.mark.parametrize(
    "entry_point",
    [
        "csv_export2",
        "overburden_timeshift",
        "design2params",
        "gendata_rft",
        "design_kw",
        "fm_pyscal",
        "semeio_stea",
    ],
)
def test_console_scripts_help(script_runner, entry_point):
    """
    This test is a quick test that the scripts are installed correctly,
    by checking that we call call --help (which all these have). Semeio
    must be installed for them to pass.
    """
    ret = script_runner.run(entry_point, "--help")
    assert ret.success



#!/usr/bin/env python
import os
import sys

iens = None
if len(sys.argv) > 1:
    iens = int(sys.argv[1])

numbers = [1, 2, 3, 4, 5, 6]
if iens in numbers:
    random_report_step = (numbers.index(iens) % 3) + 1
    os.remove("perlin_%d.txt" % random_report_step)

    with open("perlin_fail.status", "w") as f:
        f.write("Deleted report step: %d" % random_report_step)

else:
    with open("perlin_fail.status", "w") as f:
        f.write("Did nothing!")



import filecmp
import logging
import os
from distutils.dir_util import copy_tree

import pandas as pd
import pytest

from semeio.jobs.design2params import design2params
from semeio.jobs.design_kw.design_kw import extract_key_value
from semeio._exceptions.exceptions import ValidationError


@pytest.fixture
def input_data(tmpdir):
    data_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "data")
    copy_tree(data_dir, tmpdir.strpath)

    cwd = os.getcwd()
    tmpdir.chdir()

    yield

    os.chdir(cwd)


@pytest.mark.usefixtures("input_data")
@pytest.mark.parametrize(
    "test_file, expected_file",
    [
        (design2params._PARAMETERS_TXT, "refparameters.txt"),
        (design2params._DESIGN_MATRIX_TXT, "refdesignmatrix.txt"),
        (design2params._DESIGN_PARAMETERS_TXT, "refdesignparameters.txt"),
    ],
)
def test_run(test_file, expected_file):
    design2params.run(
        1,
        "design_matrix.xlsx",
        "DesignSheet01",
        "DefaultValues",
        design2params._PARAMETERS_TXT,
        log_level=logging.DEBUG,
    )

    assert filecmp.cmp(test_file, expected_file)

    with open(design2params._TARGET_FILE_TXT, "r") as status_file:
        status = status_file.read()
        assert status == "OK\n"


@pytest.mark.usefixtures("input_data")
@pytest.mark.parametrize(
    "test_file, expected_file",
    [
        ("new_parameters.txt", "refparameters_when_missing.txt"),
        (design2params._DESIGN_MATRIX_TXT, "refdesignmatrix.txt"),
        (design2params._DESIGN_PARAMETERS_TXT, "refparameters_when_missing.txt"),
    ],
)
def test_run_with_no_parameters_txt(test_file, expected_file):
    design2params.run(
        1,
        "design_matrix.xlsx",
        "DesignSheet01",
        "DefaultValues",
        "new_parameters.txt",
        log_level=logging.DEBUG,
    )

    assert filecmp.cmp(test_file, expected_file)

    with open(design2params._TARGET_FILE_TXT, "r") as status_file:
        status = status_file.read()
        assert status == "OK\n"


@pytest.mark.usefixtures("input_data")
@pytest.mark.parametrize(
    "test_file, expected_file",
    [
        (design2params._PARAMETERS_TXT, "refparameters_w_default.txt"),
        (design2params._DESIGN_MATRIX_TXT, "refdesignmatrix.txt"),
        (design2params._DESIGN_PARAMETERS_TXT, "refdesignparameters_w_default.txt"),
    ],
)
def test_run_with_default(test_file, expected_file):
    design2params.run(
        1,
        "design_matrix_tampered_defaults.xlsx",
        "DesignSheet01",
        "DefaultValues",
        design2params._PARAMETERS_TXT,
        log_level=logging.DEBUG,
    )

    assert filecmp.cmp(test_file, expected_file)

    with open(design2params._TARGET_FILE_TXT, "r") as status_file:
        status = status_file.read()
        assert status == "OK\n"


@pytest.mark.usefixtures("input_data")
@pytest.mark.parametrize(
    "test_file, expected_file",
    [
        (design2params._PARAMETERS_TXT, "refparameters_w_spaces.txt"),
        (design2params._DESIGN_MATRIX_TXT, "refdesignmatrix_w_spaces.txt"),
        (design2params._DESIGN_PARAMETERS_TXT, "refdesignparameters_w_spaces.txt"),
    ],
)
def test_run_with_spaces_in_cells(test_file, expected_file):
    design2params.run(
        1,
        "design_matrix_cells_with_spaces.xlsx",
        "DesignSheet01",
        "DefaultValues",
        design2params._PARAMETERS_TXT,
        log_level=logging.DEBUG,
    )

    assert filecmp.cmp(test_file, expected_file)

    with open(design2params._TARGET_FILE_TXT, "r") as status_file:
        status = status_file.read()
        assert status == "OK\n"


@pytest.mark.usefixtures("input_data")
@pytest.mark.parametrize("realization_id", [(0), (1), (15), (24)])
def test_runs_different_reals_all_ok(realization_id):
    design2params.run(
        realization_id,
        "design_matrix_tampered_defaults.xlsx",
        parametersfilename=design2params._PARAMETERS_TXT,
        log_level=logging.DEBUG,
    )

    with open(design2params._TARGET_FILE_TXT, "r") as status_file:
        status = status_file.read()
        assert status == "OK\n"


def test_empty_defaults(tmpdir):
    # pylint: disable=abstract-class-instantiated
    tmpdir.chdir()
    write_design_xlsx("design_matrix.xlsx")

    parsed_defaults = design2params._read_defaultssheet(
        "design_matrix.xlsx", "DefaultValues"
    )
    assert parsed_defaults.empty
    assert (parsed_defaults.columns == ["keys", "defaults"]).all()


def test_one_column_defaults(tmpdir):
    tmpdir.chdir()
    write_design_xlsx("design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[["foo"]]))

    with pytest.raises(SystemExit):
        design2params._read_defaultssheet("design_matrix.xlsx", "DefaultValues")


def test_three_column_defaults(tmpdir):
    """
    A deprecation warning will be emitted in this test.

    Change this test later to assert the code will fail hard.
    """
    # pylint: disable=abstract-class-instantiated
    tmpdir.chdir()
    write_design_xlsx(
        "design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[["foo", "bar", "com"]])
    )

    parsed_defaults = design2params._read_defaultssheet(
        "design_matrix.xlsx", "DefaultValues"
    )
    assert len(parsed_defaults) == 1
    assert (parsed_defaults.columns == ["keys", "defaults"]).all()


def test_run_realization_not_exist(input_data):
    with pytest.raises(SystemExit):
        design2params.run(
            1000,
            "design_matrix.xlsx",
            "DesignSheet01",
            "DefaultValues",
            design2params._PARAMETERS_TXT,
            log_level=logging.DEBUG,
        )


def test_open_excel_file_not_existing(input_data):
    with pytest.raises(SystemExit):
        design2params._read_excel("not_existing.xlsx", "DesignSheet01")


def test_open_excel_file_not_xls(input_data):
    with pytest.raises(SystemExit):
        design2params._read_excel("parameters.txt", "DesignSheet01")


def test_open_excel_file_header_missing(input_data):
    with pytest.raises(
        ValidationError,
        match="Design matrix not valid, error: Column headers not present in column",
    ):
        design2params.run(
            1,
            "design_matrix_missing_header.xlsx",
            "DesignSheet01",
            "DefaultValues",
            design2params._PARAMETERS_TXT,
            log_level=logging.DEBUG,
        )


def test_open_excel_file_value_missing(input_data, caplog):
    """Check that we get SystemExit only for those realizations
    affected by missing cell values

    (all realizations see warnings for empty cells in other realizations)
    """
    reals_with_missing_values = [7, 11]
    for real in range(0, 25):
        if real in reals_with_missing_values:
            with pytest.raises(SystemExit):
                design2params.run(
                    real,
                    "design_matrix_missing_value.xlsx",
                    "DesignSheet01",
                    "DefaultValues",
                    design2params._PARAMETERS_TXT,
                    log_level=logging.DEBUG,
                )
        else:
            design2params.run(
                real,
                "design_matrix_missing_value.xlsx",
                "DesignSheet01",
                "DefaultValues",
                design2params._PARAMETERS_TXT,
                log_level=logging.DEBUG,
            )
    assert "Realization 7, column RMS_SEED" in caplog.text
    assert "Realization 11, column MULTFLT_F1" in caplog.text


@pytest.mark.parametrize(
    "exist_params, design_m, expected_warning",
    [
        ("NAMESPACE:FOO 3", {"FOO": 5}, None),
        ("FOO 3", {"FOO": 4}, "Parameter FOO already exists"),
        ("FOO 3", {"FOO": 3}, None),  # Silently ignore this situation
        ("", {"FOO": 3}, None),
        ("FOO 4\nBAR 5", {"BAR": 6}, "Parameter BAR already exists"),
        ("FOO 4\nBAR 5", {"FOO": 4}, None),
        ("FOO 4\nBAR 5", {"FOO": 5}, "Parameter FOO already exists"),
        ("FOO 4\nBAR 5", {"FOO": 4, "BAR": 6}, "Parameter BAR already exists"),
    ],
)
def test_existing_parameterstxt(
    exist_params, design_m, expected_warning, tmpdir, caplog
):
    """Test warnings emitted when the file parameters.txt is prefilled"""
    tmpdir.chdir()

    params_file = "parameters.txt"
    with open(params_file, "w") as file_h:
        file_h.write(exist_params)

    designsheet_df = pd.DataFrame.from_records(data=[design_m])
    designsheet_df.insert(0, "REAL", [0])

    write_design_xlsx("design_matrix.xlsx", designdf=designsheet_df)

    design2params.run(
        0,
        "design_matrix.xlsx",
        parametersfilename=params_file,
        log_level=logging.DEBUG,
    )

    if expected_warning:
        assert expected_warning in caplog.text
    else:
        assert "already exists" not in caplog.text


def test_open_excel_file_wrong_defaults(input_data):
    with pytest.raises(SystemExit):
        design2params.run(
            1,
            "design_matrix_missing_value.xlsx",
            "DesignSheet01",
            "DefaultValuesWRONGNAME",
            design2params._PARAMETERS_TXT,
            log_level=logging.DEBUG,
        )


@pytest.mark.parametrize(
    "cellvalue, expected_parameters_str",
    [
        ("TRUE", "TRUE"),
        ("FALSE", "FALSE"),
        ("True", "True"),
        (True, "True"),
        (False, "False"),
        (0, "0"),
        (0.000, "0"),
        (1, "1"),
        ("1", "1"),
        ("1.0", "1.0"),
        (1.0, "1"),  # This difference is related to to_excel()
        ("æøå", "æøå"),
        (1e-1, "0.1"),
        (1e-4, "0.0001"),
        (1e-5, "1e-05"),
        (1e-16, "1e-16"),
        (1e3, "1000"),
        (1e10, "10000000000"),
    ],
)
def test_single_cell_values(cellvalue, expected_parameters_str, tmpdir):
    """Test how certain single values go through the Excel input sheets
    and all the way to parameters.txt"""
    tmpdir.chdir()
    write_design_xlsx(
        "design_matrix.xlsx",
        designdf=pd.DataFrame(columns=["REAL", "SOMEKEY"], data=[[0, cellvalue]]),
    )

    params_file = "parameters.txt"
    design2params.run(
        0,
        "design_matrix.xlsx",
        parametersfilename=params_file,
        log_level=logging.DEBUG,
    )
    with open(params_file) as p_file:
        params_lines = p_file.readlines()
    key_vals = extract_key_value(params_lines)
    assert key_vals["SOMEKEY"] == expected_parameters_str


@pytest.mark.parametrize(
    "cellvalues, expected_parameters_strs",
    [
        (["TRUE", "True"], ["TRUE", "True"]),
        (["TRUE", 1], ["TRUE", "1"]),
        (["True", 1.0], ["True", "1"]),
        (["True", "1.0"], ["True", "1.0"]),
        ([0, "1.0"], ["0", "1.0"]),
        ([1, 1.2], ["1", "1.2"]),
        (["æ", 1e10], ["æ", "10000000000"]),
    ],
)
def test_pair_cell_values(cellvalues, expected_parameters_strs, tmpdir):
    """Test how a pair of values, one for each realization, go through
    the Excel input sheets and all the way to parameters.txt.

    This is to ensure that differing datatypes in a single Excel columns does
    not affect individual values."""
    tmpdir.chdir()

    write_design_xlsx(
        "design_matrix.xlsx",
        designdf=pd.DataFrame(
            columns=["REAL", "SOMEKEY"], data=[[0, cellvalues[0]], [1, cellvalues[1]]]
        ),
    )

    params_0 = "parameters0.txt"
    design2params.run(
        0,
        "design_matrix.xlsx",
        parametersfilename=params_0,
        log_level=logging.DEBUG,
    )
    params_1 = "parameters1.txt"
    design2params.run(
        1,
        "design_matrix.xlsx",
        parametersfilename=params_1,
        log_level=logging.DEBUG,
    )
    with open(params_0) as p_file:
        params_lines_0 = p_file.readlines()
    with open(params_1) as p_file:
        params_lines_1 = p_file.readlines()
    key_vals_real0 = extract_key_value(params_lines_0)
    key_vals_real1 = extract_key_value(params_lines_1)
    assert key_vals_real0["SOMEKEY"] == expected_parameters_strs[0]
    assert key_vals_real1["SOMEKEY"] == expected_parameters_strs[1]


@pytest.mark.parametrize(
    "paramnames",
    (
        [
            ["REAL", "SOMEKEY "],
            ["REAL", " SOMEKEY"],
            ["REAL", " SOMEKEY "],
            ["REAL", "\tSOMEKEY"],
            ["REAL", "SOMEKEY\t"],
            ["REAL", "SOMEKEY\n"],
        ]
    ),
)
def test_headers_trailing_whitespace(paramnames, tmpdir):
    """Parameter names can have un-noticed trailing whitespace in Excel.

    This can happen both in the design- and the defaultssheet.

    This should error hard as it has no believed use-case and only
    creates user confusion.
    """
    tmpdir.chdir()

    write_design_xlsx(
        "design_matrix.xlsx",
        designdf=pd.DataFrame(columns=paramnames, data=[[0, "foo"]]),
    )
    write_design_xlsx(
        "design_matrix_onlydefaults.xlsx",
        defaultsdf=pd.DataFrame(data=[[paramnames[1], "bar"]]),
    )

    with pytest.raises(SystemExit, match="whitespace"):
        design2params.run(
            0,
            "design_matrix.xlsx",
        )
    with pytest.raises(SystemExit, match="whitespace"):
        design2params.run(
            0,
            "design_matrix_onlydefaults.xlsx",
            parametersfilename="parameters_onlydefaults.txt",
        )


@pytest.mark.parametrize("paramname", design2params.DENYLIST)
def test_denylist_designsheet(paramname, tmpdir):
    """Some parameter names are reserved, ensure we error hard if found."""
    tmpdir.chdir()

    write_design_xlsx(
        "design_matrix.xlsx",
        designdf=pd.DataFrame(columns=["REAL", paramname], data=[[0, "foo"]]),
    )
    with pytest.raises(SystemExit, match="not allowed"):
        design2params.run(
            0,
            "design_matrix.xlsx",
        )


@pytest.mark.parametrize("paramname", design2params.DENYLIST_DEFAULTS)
def test_denylist_defaults(paramname, tmpdir):
    """Some parameter names are reserved, ensure we error hard if found.

    The defaults sheet can have more denied parameters than the designsheet,
    in particular the "REAL" column."""
    tmpdir.chdir()
    write_design_xlsx(
        "design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[[paramname, "bar"]])
    )
    with pytest.raises(SystemExit, match="not allowed"):
        design2params.run(
            0,
            "design_matrix.xlsx",
        )


@pytest.mark.parametrize("paramname", [1, 1.0])
def test_invalid_pandas_header_error(tmpdir, paramname):
    tmpdir.chdir()

    write_design_xlsx(
        "design_matrix.xlsx",
        designdf=pd.DataFrame(columns=["REAL", paramname], data=[[0, "foo"]]),
    )

    expected_error = "Invalid value in design matrix header"
    with pytest.raises(ValidationError, match=expected_error):
        design2params.run(
            0,
            "design_matrix.xlsx",
        )


@pytest.mark.parametrize("paramname", [1, 1.0])
def test_denylist_expected_error(paramname, tmpdir):
    tmpdir.chdir()
    write_design_xlsx(
        "design_matrix.xlsx", defaultsdf=pd.DataFrame(data=[[paramname, "bar"]])
    )
    with pytest.raises(SystemExit, match="does not exist in design matrix"):
        design2params.run(
            0,
            "design_matrix.xlsx",
        )


@pytest.mark.parametrize(
    "paramset", [["REAL", "REAL"], ["REAL", "REAL", "REAL"], ["FOO", "FOO"]]
)
def test_duplicated_designcolumns(paramset, tmpdir, caplog):
    """If the excel sheet contains duplicated column headers, pandas will
    happily read it but modify column names. This is most likely a user error,
    and a warning is issued"""

    tmpdir.chdir()
    write_design_xlsx(
        "design_matrix.xlsx",
        designdf=pd.DataFrame(
            columns=["REAL"] + paramset, data=[[0] + ["foo"] * len(paramset)]
        ),
    )

    design2params.run(
        0,
        "design_matrix.xlsx",
    )
    assert "are probably duplicated" in caplog.text


def write_design_xlsx(
    filename,
    designdf=None,
    defaultsdf=None,
    designsheetname="DesignSheet01",
    defaultssheetname="DefaultValues",
):
    """Generate an XLS file with a design sheet and a defaults sheet.

    Args:
        filename (str)
        designdf (pd.DataFrame): Design data. Empty data will be written if None
        defaultsdf (pd.DataFrame): defaults data. Empty data will be written if None
        designsheetname (str): sheet-name in xlsx file. Defaulted if not provided
        defaultssheetname (str): sheet-name in xlsx file. Defaulted if not provided
    """
    # pylint: disable=abstract-class-instantiated
    writer = pd.ExcelWriter(filename)
    if designdf is not None:
        designdf.to_excel(writer, sheet_name=designsheetname, index=False)
    else:
        # Write empty sheet.
        pd.DataFrame().to_excel(writer, sheet_name=designsheetname, index=False)

    if defaultsdf is not None:
        defaultsdf.to_excel(
            writer, sheet_name=defaultssheetname, index=False, header=None
        )
    else:
        # Write empty sheet.
        pd.DataFrame().to_excel(
            writer, sheet_name=defaultssheetname, index=False, header=None
        )
    writer.save()



from setuptools import setup, find_packages


setup(
    name="semeio",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    use_scm_version={"write_to": "semeio/version.py"},
    author="Equinor ASA",
    author_email="fg_sib-scout@equinor.com",
    url="https://github.com/equinor/semeio",
    description="Jobs and workflow jobs for Ert.",
    packages=find_packages(include=["semeio*"]),
    entry_points={
        "ert": [
            "semeio_jobs = semeio.hook_implementations.jobs",
            "SpearmanCorrelation = semeio.workflows.spearman_correlation_job.spearman_correlation",  # noqa
            "MisfitPreprocessor = semeio.workflows.misfit_preprocessor.misfit_preprocessor",  # noqa
            "CorrelatedObsScaling = semeio.workflows.correlated_observations_scaling.cos",  # noqa
            "CsvExport2Job = semeio.workflows.csv_export2.csv_export2",
        ],
        "console_scripts": [
            "csv_export2=semeio.workflows.csv_export2.csv_export2:cli",
            "overburden_timeshift=semeio.jobs.scripts.overburden_timeshift:main_entry_point",  # noqa
            "design2params=semeio.jobs.scripts.design2params:main_entry_point",
            "gendata_rft=semeio.jobs.scripts.gendata_rft:main_entry_point",
            "design_kw=semeio.jobs.scripts.design_kw:main_entry_point",
            "fm_pyscal=semeio.jobs.scripts.fm_pyscal:main_entry_point",
            "semeio_stea=semeio.jobs.scripts.fm_stea:main_entry_point",
        ],
    },
    license="GPL-3.0",
    platforms="any",
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        "ecl",
        "ert",
        "equinor-libres",
        "configsuite>=0.6",
        "numpy",
        "pandas>0.24",
        "scipy",
        "xlrd<2",
        "stea",
        "pyscal>=0.4.0",
        "fmu-ensemble",
        "segyio",
    ],
    include_package_data=True,
)



import numpy as np

from scipy.interpolate import CloughTocher2DInterpolator

import segyio
from segyio import TraceField


class OTSVelSurface(object):
    def __init__(self, res_surface, vcube):
        """
        Create a surface where the timeshift can be calculated.

        Read a reservoir
        """

        self._x = None
        self._y = None
        self._z = None

        self._nx = None
        self._ny = None

        self._z3d = None
        self._dt = None

        self._map_reservoir_surface_to_velocity(res_surface, vcube)

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def z(self):
        return self._z

    @property
    def nx(self):
        return self._nx

    @property
    def ny(self):
        return self._ny

    @property
    def z3d(self):
        return self._z3d

    @property
    def dt(self):
        return self._dt

    def __len__(self):
        return len(self.x)

    def __str__(self):

        s = "x: " + str(self.x)
        s += "\ny: " + str(self.y)
        s += "\nz: " + str(self.z)
        s += "\ndt=" + str(self.dt)
        s += "\nz3d=" + str(self.z3d)
        return s

    def _read_velocity(self, vcube, cell_corners):
        """
        Read velocity from segy file. Upscale.
        :param vcube:
        :param cell_corners:
        :return:
        """

        with segyio.open(vcube) as f:
            x = np.empty(f.tracecount, dtype=np.float64)
            y = np.empty(f.tracecount, dtype=np.float64)
            dt = segyio.dt(f) / 1e6
            nt = len(f.samples)

            for i, h in enumerate(f.header):
                scalar = h[TraceField.SourceGroupScalar]
                if scalar < 0:
                    scalar = -1.0 / scalar
                x[i] = h[TraceField.CDP_X] * scalar
                y[i] = h[TraceField.CDP_Y] * scalar

            ny = len(f.xlines)
            nx = len(f.ilines)

            # memory issue might happen if volume becomes too large
            traces = f.trace.raw[:]

            if f.sorting == segyio.TraceSortingFormat.INLINE_SORTING:
                x = x.reshape(ny, nx)
                y = y.reshape(ny, nx)
                traces = traces.reshape(ny, nx, nt)
                x = np.transpose(x)
                y = np.transpose(y)
                traces = np.transpose(traces, (1, 0, 2))
            elif f.sorting == segyio.TraceSortingFormat.CROSSLINE_SORTING:
                x = x.reshape(nx, ny)
                y = y.reshape(nx, ny)
                traces = traces.reshape(nx, ny, nt)
            else:
                raise RuntimeError

        x, y, traces, nt, dt = self._upscale_velocity(
            cell_corners, x, y, traces, nt, dt
        )

        return x, y, traces, nt, dt

    @staticmethod
    def _upscaling_size_stepping(res_corners, axis, vel_axis):
        """
        Upscales axis of velocity.

        :param res_corners: All corner points of reservoir model, along @axis
        :param axis: Gives which axis is being upscaled. 0 for x, 1 for y
        :param vel_axis: axis of model, x or y given by @axis
        :return:
        """
        res_n = res_corners.shape[axis]
        # average extent of the grid in axis direction divided by resolution
        # gives us average extent of one voxel
        size = np.mean(np.max(res_corners, axis) - np.min(res_corners, axis)) / res_n
        # the number of grid voxels that can be fit into the segy volume
        # defined by CDP_X and CDP_Y
        nn = np.ceil(
            np.mean(np.max(vel_axis, axis) - np.min(vel_axis, axis)) / size
        ).astype(np.int)

        n = vel_axis.shape[axis]
        ups = np.floor((n - 1) / nn).astype(np.int)
        # always only upscaling
        if ups < 1:
            ups = 1
            nn = n

        return nn, ups

    def _upscale_velocity(self, res_corners, x_vel, y_vel, traces, nt, dt):
        # resample to a new grid size based on the grid size
        nxx, upsx = self._upscaling_size_stepping(res_corners[:, :, 0], 0, x_vel)

        nyy, upsy = self._upscaling_size_stepping(res_corners[:, :, 1], 1, y_vel)

        ntt = int(np.floor(nt / (16e-3 / dt)))
        upst = int(np.floor((nt - 1) / ntt))
        if upst == 0:
            upst = 1
            ntt = nt
        dt = dt * upst
        # skip positions if needed
        x = x_vel[0 : (nxx - 1) * upsx + 1 : upsx, 0 : (nyy - 1) * upsy + 1 : upsy]
        y = y_vel[0 : (nxx - 1) * upsx + 1 : upsx, 0 : (nyy - 1) * upsy + 1 : upsy]
        # upscale traces if needed, i.e., ntt > nt
        # skip samples if needs, ups >= 2
        traces_upscaled = traces[
            0 : (nxx - 1) * upsx + 1 : upsx,
            0 : (nyy - 1) * upsy + 1 : upsy,
            0 : (ntt - 1) * upst + 1 : upst,
        ]

        return x, y, traces_upscaled, nt, dt

    def _map_reservoir_surface_to_velocity(self, res_surface, vcube):
        """
        Interpolates reservoir top surface to velocity grid
        """
        # downsample segy if segy resultion of CDP and
        # sample rate is higher than of the Eclgrid
        x, y, traces, nt, self._dt = self._read_velocity(
            vcube, res_surface.cell_corners
        )
        # this is some clever integration of traces using
        # averaging between two samples?
        vel_t_int = np.zeros_like(traces)
        vel_t_int[:, :, 1:] = (traces[:, :, 0:-1] + traces[:, :, 1:]) / 2
        # cumulative sum of each trace and reshape to 2D
        # array of size (num_trace, num_samples)
        self._z3d = np.cumsum(vel_t_int * self._dt / 2, 2).reshape(
            -1, vel_t_int.shape[-1]
        )
        # this creates interploation functions over existing surface / grid
        ip = CloughTocher2DInterpolator((res_surface.x, res_surface.y), res_surface.z)
        # interpolate over our vel field given by pos x and y
        z = ip(x, y)
        # z gives a new depth for segy CDP pos given the grid inerface surface

        # So far we have downsampled the segy file to correspond
        # with the resolution of the grid.
        # The x,y values corresponds to the remaining segy positions.
        # furthermore the z values are the interpolated values that
        # match the top layer of the active cells in the grid file.
        self._nx, self._ny = x.shape
        self._x = x.reshape(-1)
        self._y = y.reshape(-1)
        self._z = z.reshape(-1)



import numpy as np
from semeio.jobs.overburden_timeshift.ots_res_surface import OTSResSurface
from ecl.grid import EclGrid, EclGridGenerator
import os
import pytest


def get_source_ert(grid):
    x = np.zeros(grid.getNumActive(), np.float64)
    y = np.zeros(grid.getNumActive(), np.float64)
    z = np.zeros(grid.getNumActive(), np.float64)
    v = np.zeros(grid.getNumActive(), np.float64)
    for i in range(grid.getNumActive()):
        (x[i], y[i], z[i]) = grid.get_xyz(active_index=i)
        v[i] = grid.cell_volume(active_index=i)

    return x, y, z, v


def test_res_surface(ots_tmpdir_enter):
    eclcase_dir = ots_tmpdir_enter
    grid_path = os.path.join(eclcase_dir, "NORNE_ATW2013.EGRID")
    grid = EclGrid(grid_path, apply_mapaxes=False)

    rec = OTSResSurface(grid=grid, above=100)

    assert rec.nx == 46
    assert rec.ny == 112

    assert 453210.38 == pytest.approx(np.min(rec.x))
    assert 465445.16 == pytest.approx(np.max(rec.x))

    assert 7316018.5 == pytest.approx(np.min(rec.y))
    assert 7330943.5 == pytest.approx(np.max(rec.y))

    assert 2177.6484 == pytest.approx(np.min(rec.z))
    assert 3389.567 == pytest.approx(np.max(rec.z))


def test_surface(tmpdir):
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=[0, 0, 0, 0, 1, 1, 1, 1]
    )

    surface = OTSResSurface(grid=grid)

    z = np.ones(4) * 100
    assert np.all(z == surface.z)


def test_surface_above(tmpdir):
    grid = EclGridGenerator.createRectangular(
        dims=(2, 2, 2), dV=(100, 100, 100), actnum=[0, 0, 0, 0, 1, 1, 1, 1]
    )

    surface = OTSResSurface(grid=grid, above=10)

    z = np.ones(4) * (100 - 10)
    assert np.all(z == surface.z)



import os
import pytest
import shutil

from tests import test_data

TEST_NORNE_DIR = os.path.realpath(os.path.join(test_data.__path__[0], "norne"))


@pytest.fixture(scope="session")
def _setup_ots_tmppath(tmpdir_factory):
    # for the sake of python2.7 the path needs to be converted to string
    tmp_path = tmpdir_factory.mktemp("norne", numbered=False).join("test-data").strpath
    shutil.copytree(TEST_NORNE_DIR, tmp_path)
    return tmp_path


# use this fixture for setting up norne data and entering
# another folder while having handle to norne data dir
@pytest.fixture
def ots_tmpdir_enter(_setup_ots_tmppath, tmpdir):
    tmp_path = _setup_ots_tmppath
    with tmpdir.as_cwd():
        yield tmp_path



import numpy as np


class OTSResSurface(object):
    def __init__(self, grid, above=0):
        """
        Create a surface from a reservoir grid.

        The surface will be located given meters above most shallow active cell

        :param grid: Reservoir grid
        :param above: Scalar: meters above most shallow active cell
        """

        self._grid = grid
        self._x = None
        self._y = None
        self._z = None
        self._nx = None
        self._ny = None

        self._calculate_surface(grid, above)

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def z(self):
        return self._z

    @property
    def nx(self):
        return self._nx

    @property
    def ny(self):
        return self._ny

    def __len__(self):
        return len(self.x)

    def __str__(self):
        return "x: {}\ny: {}\nz: {}".format(self.x, self.y, self.z)

    @property
    def cell_corners(self):
        return self._get_top_corners()

    def _calculate_surface(self, grid, above):
        # calculate average from top face vecrtices
        # from unstructured grid as an interface between active and inactive cells
        nx = grid.getNX()
        ny = grid.getNY()
        nz = grid.getNZ()

        x = np.empty(shape=(nx * ny), dtype=np.float32)
        y = np.empty(shape=(nx * ny), dtype=np.float32)
        z = np.empty(shape=(nx * ny), dtype=np.float32)
        for i in range(nx):
            for j in range(ny):
                _k = nz - 1
                for k in range(nz):
                    if grid.active(ijk=(i, j, k)):
                        _k = k
                        break
                ijk = (i, j, _k)

                top = [grid.getCellCorner(c, ijk=ijk) for c in range(0, 4)]
                pos = i * ny + j
                x[pos] = sum(val[0] for val in top) / 4.0
                y[pos] = sum(val[1] for val in top) / 4.0
                z[pos] = sum(val[2] for val in top) / 4.0

        z -= above

        self._x = x
        self._y = y
        self._z = z
        self._nx = nx
        self._ny = ny

    def _get_top_corners(self):
        grid = self._grid
        nx = grid.getNX() + 1
        ny = grid.getNY() + 1
        top_corners = np.empty(shape=(nx, ny, 3), dtype=np.float32)
        for i in range(nx):
            for j in range(ny):
                # getNodePos should mostly return top-right vertex position of the node
                top_corners[i, j] = grid.getNodePos(i, j, 0)

        return top_corners



import pytest

from ecl.grid import EclGridGenerator
from semeio.jobs.overburden_timeshift.ots_vel_surface import OTSVelSurface
from semeio.jobs.overburden_timeshift.ots_res_surface import OTSResSurface
import segyio
from ots_util import create_segy_file


@pytest.fixture()
def setup_spec(tmpdir):
    spec = segyio.spec()
    spec.format = 5
    spec.sorting = 2
    spec.samples = range(0, 40, 4)
    spec.ilines = range(2)
    spec.xlines = range(2)
    yield spec


def test_create(setup_spec):
    spec = setup_spec
    vel = "TEST.segy"
    create_segy_file(vel, spec)
    grid = EclGridGenerator.createRectangular(dims=(2, 2, 2), dV=(100, 100, 100))
    res_surface = OTSResSurface(grid=grid, above=0)

    OTSVelSurface(res_surface, vel)


def test_surface(setup_spec):
    spec = setup_spec
    vel = "TEST.segy"
    int_val = [50, 150]
    create_segy_file(vel, spec, xl=int_val, il=int_val, cdp_x=int_val, cdp_y=int_val)
    grid = EclGridGenerator.createRectangular(dims=(2, 2, 2), dV=(100, 100, 100))
    res_surface = OTSResSurface(grid=grid, above=0)
    ots_s = OTSVelSurface(res_surface, vel)

    assert ots_s.x[0] == 50
    assert ots_s.y[0] == 50
    assert ots_s.z[0] == 0
    assert ots_s.nx == 2
    assert ots_s.ny == 2


def test_z3d(setup_spec):
    spec = setup_spec
    vel = "TEST.segy"
    int_val = [50, 150]
    create_segy_file(vel, spec, xl=int_val, il=int_val, cdp_x=int_val, cdp_y=int_val)
    grid = EclGridGenerator.createRectangular(dims=(2, 2, 2), dV=(100, 100, 100))
    res_surface = OTSResSurface(grid=grid, above=0)
    ots_s = OTSVelSurface(res_surface, vel)

    assert (4, 2) == ots_s.z3d.shape



#! /usr/bin/env python

if __name__ == "__main__":
    files = {
        "PERLIN_1": "perlin_1.txt",
        "PERLIN_2": "perlin_2.txt",
        "PERLIN_3": "perlin_3.txt",
    }

    with open("aggregated.txt", "w") as output_file:
        sum_of_sum = 0.0
        for key in files:
            sum = 0.0  # pylint: disable=redefined-builtin
            with open(files[key], "r") as input_file:
                sum += float(input_file.readline())
            sum_of_sum += sum
            output_file.write("%s %f\n" % (key, sum))

        if sum_of_sum < 0:
            state = "Negative"
        elif abs(sum_of_sum) < 0.00000001:
            state = "Zero"
        else:
            state = "Positive"

        output_file.write("STATE %s" % state)
